"""
Classes for working with position specific matrices.
"""
from copy import copy

import numpy
from numpy import (
    float32,
    int16,
    log2,
    maximum,
    nan,
    newaxis,
    ones,
    zeros
)

from . import _pwm


class BaseMatrix(object):
    """
    Base class for position specific matrices.
    """

    def __init__(self, alphabet=None, sorted_alphabet=None, char_to_index=None, values=None):
        self.alphabet = alphabet
        self.sorted_alphabet = sorted_alphabet
        self.char_to_index = char_to_index
        self.values = values

    @classmethod
    def from_rows(Class, alphabet, rows):
        """
        Create a new matrix for a sequence over alphabet `alphabet` taking
        values from `rows` which is a list whose length is the width of the
        matrix, and whose elements are lists of values associated with each
        character (in the order those characters appear in alphabet).
        """
        # Sorted alphabet
        sorted_alphabet = sorted(alphabet)
        # Character to index mapping (initialized to -1)
        char_to_index = zeros((256), int16) - 1
        for i, ch in enumerate(sorted_alphabet):
            char_to_index[ord(ch)] = i
        # Array
        values = zeros((len(rows), len(alphabet)), float32)
        for i, row in enumerate(rows):
            assert len(row) == len(alphabet)
            for ch, val in zip(alphabet, row):
                values[i, char_to_index[ord(ch)]] = val
        # Matrix
        matrix = Class()
        matrix.alphabet = alphabet
        matrix.sorted_alphabet = sorted_alphabet
        matrix.char_to_index = char_to_index
        matrix.values = values
        return matrix

    @classmethod
    def create_from_other(Class, other, values=None):
        """
        Create a new Matrix with attributes taken from `other` but with the
        values taken from `values` if provided
        """
        m = Class()
        m.alphabet = other.alphabet
        m.sorted_alphabet = other.sorted_alphabet
        m.char_to_index = other.char_to_index
        if values is not None:
            m.values = values
        else:
            m.values = other.values
        return m

    @property
    def width(self):
        """
        Return the width (size along the sequence axis) of this matrix.
        """
        return self.values.shape[0]

    def reverse_complement(self):
        """
        Create the reverse complement of this matrix. The result probably
        only makese sense if the alphabet is that of DNA ('A','C','G','T').
        """
        rval = copy(self)
        # Conveniently enough, reversing rows and columns is exactly what we
        # want, since this results in A swapping with T and C swapping with G.
        rval.values = self.values[::-1, ::-1].copy()
        return rval


class FrequencyMatrix(BaseMatrix):
    """
    A position specific count/frequency matrix.
    """

    DEFAULT_CORRECTION = 0.0000000001
    """
    Default value to use for correcting when dealing with counts of zero,
    chosen to produce scoring matrices that are the same as produced by CREAD.
    """

    def to_logodds_scoring_matrix(self, background=None, correction=DEFAULT_CORRECTION):
        """
        Create a standard logodds scoring matrix.
        """
        alphabet_size = len(self.alphabet)
        if background is None:
            background = ones(alphabet_size, float32) / alphabet_size
        # Row totals as a one column array
        totals = numpy.sum(self.values, 1)[:, newaxis]
        values = log2(maximum(self.values, correction)) - log2(totals) - log2(maximum(background, correction))
        return ScoringMatrix.create_from_other(self, values.astype(float32))

    def to_stormo_scoring_matrix(self, background=None):
        """
        Create a scoring matrix from this count matrix using the method from:

        Hertz, G.Z. and G.D. Stormo (1999). Identifying DNA and protein patterns with statistically
        significant alignments of multiple sequences. Bioinformatics 15(7): 563-577.
        """
        alphabet_size = len(self.alphabet)
        if background is None:
            background = ones(alphabet_size, float32) / alphabet_size
        # Row totals as a one column array
        totals = numpy.sum(self.values, 1)[:, newaxis]
        values = log2(self.values + background) - log2(totals + 1) - log2(background)
        return ScoringMatrix.create_from_other(self, values.astype(float32))


class ScoringMatrix(BaseMatrix):
    """
    A position specific matrix containing values that are suitable for
    scoring a sequence.
    """

    def score_string(self, string):
        """
        Score each valid position in `string` using this scoring matrix.
        Positions which were not scored are set to nan.
        """
        rval = zeros(len(string), float32)
        rval[:] = nan
        _pwm.score_string(self.values, self.char_to_index, string, rval)
        return rval

    def score_string_with_gaps(self, string):
        """
        Score each valid position in `string` using this scoring matrix.
        Positions which were not scored are set to nan. Gap characters are
        ignored (matrices score across them).
        """
        rval = zeros(len(string), float32)
        rval[:] = nan
        _pwm.score_string_with_gaps(self.values, self.char_to_index, string, rval)
        return rval



"""
Support for scores in the `wiggle`_ file format used by the UCSC Genome
Browser.

The positions in the wiggle format are 1-relative, however,
the positions returned match the BED/interval format which is zero-based, half-open.

.. _wiggle: http://genome.ucsc.edu/goldenPath/help/wiggle.html
"""


def parse_header(line):
    return dict([field.split('=') for field in line.split()[1:]])


def IntervalReader(f):
    """
    Iterator yielding chrom, start, end, strand, value.
    Values are zero-based, half-open.
    Regions which lack a score are ignored.
    """
    current_chrom = None
    current_pos = None
    current_step = None

    # always for wiggle data
    strand = '+'

    mode = "bed"

    for line in f:
        if line.isspace() or line.startswith("track") or line.startswith("#") or line.startswith("browser"):
            continue
        elif line.startswith("variableStep"):
            header = parse_header(line)
            current_chrom = header['chrom']
            current_pos = None
            current_step = None
            if 'span' in header:
                current_span = int(header['span'])
            else:
                current_span = 1
            mode = "variableStep"
        elif line.startswith("fixedStep"):
            header = parse_header(line)
            current_chrom = header['chrom']
            current_pos = int(header['start']) - 1
            current_step = int(header['step'])
            if 'span' in header:
                current_span = int(header['span'])
            else:
                current_span = 1
            mode = "fixedStep"
        elif mode == "bed":
            fields = line.split()
            if len(fields) > 3:
                if len(fields) > 5:
                    yield fields[0], int(fields[1]), int(fields[2]), fields[5], float(fields[3])
                else:
                    yield fields[0], int(fields[1]), int(fields[2]), strand, float(fields[3])
        elif mode == "variableStep":
            fields = line.split()
            pos = int(fields[0]) - 1
            yield current_chrom, pos, pos + current_span, strand, float(fields[1])
        elif mode == "fixedStep":
            yield current_chrom, current_pos, current_pos + current_span, strand, float(line.split()[0])
            current_pos += current_step
        else:
            raise ValueError("Unexpected input line: %s" % line.strip())


class Reader(object):
    """
    Iterator yielding chrom, position, value.
    Values are zero-based.
    Regions which lack a score are ignored.
    """

    def __init__(self, f):
        self.file = f

    def __iter__(self):
        for chrom, start, end, strand, val in IntervalReader(self.file):
            for pos in range(start, end):
                yield chrom, pos, val



"""
Tests for `bx.seq.fasta`.
"""

import unittest

from . import fasta

test_fa = "test_data/seq_tests/test.fa"

# Same sequence data as stored in test.fa

valid_seq = "TGGAGGCATTTGTGATTCAATAGATGCAGAAAGAAACCTTCCTAGAGCTG" \
    + "GCGTTCTCTAACTAAAAGTGGAAAGTTCTGAGGAATGAGGACTGTTATAA" \
    + "ATCCCACCCCACACCGCACCTTCTCCAGGGAAGTTTCATGGCCGTGAAGA" \
    + "GGACAGAAAGTGAGAACCAAGATggaactgaataaacaagcttcacactg" \
    + "ttagtttccccatatgcttaccttcccacagatgccaaccttggaggcct" \
    + "aagaggcctagaatattatcctttgtctgatcatttctctacaaatttat" \
    + "tgttctttgttaagatgctacataagcccaaattctaaccacccctttga" \
    + "gttacccatcatcaagtttctcccatgtg"

valid_seq_len = len(valid_seq)


class FASTATestCase(unittest.TestCase):

    def test_get(self):
        fastafile = fasta.FastaFile(open(test_fa, "rb"))
        check_get(fastafile, 0, valid_seq_len)
        check_get(fastafile, 0, 40)
        check_get(fastafile, valid_seq_len - 40, 40)


def check_get(fastafile, start, len):
    assert fastafile.get(start, len) == valid_seq[start:start+len]



#!/usr/bin/env python

"""
Read a MAF from standard input and print the score of each block. It can
optionally recalculate each score using the hox70 matrix, and normalize the
score by the number of columns in the alignment.

TODO: Should be able to read an arbitrary scoring matrix.

usage: %prog [options]
   -r, --recalculate: don't use the score from the maf, recalculate (using hox70 matrix)
   -l, --lnorm: divide (normalize) score by alignment text length
"""
from __future__ import division, print_function

import sys

from bx.align import maf, score
from bx.cookbook import doc_optparse


def main():

    # Parse command line arguments
    options, args = doc_optparse.parse(__doc__)

    try:
        lnorm = bool(options.lnorm)
        recalculate = bool(options.recalculate)
    except Exception:
        doc_optparse.exit()

    hox70 = score.build_scoring_scheme("""  A    C    G    T
                                      91 -114  -31 -123
                                    -114  100 -125  -31
                                     -31 -125  100 -114
                                    -123  -31 -114   91 """, 400, 30, default=0)

    maf_reader = maf.Reader(sys.stdin)

    for m in maf_reader:
        if m.text_size == 0:
            print("NA")
            continue
        s = m.score
        # Recalculate?
        if recalculate:
            s = hox70.score_alignment(m)
        # Normalize?
        if lnorm:
            s = s / m.text_size
        # Print
        print(s)


if __name__ == "__main__":
    main()



#!/usr/bin/env python2.4
"""
Returns all positions of a maf with any pwm score > threshold
The positions are projected onto human coordinates
"""
from __future__ import print_function

import sys

from bx.align import maf as align_maf
from bx.pwm.pwm_score_maf import MafMotifScorer


def isnan(x):
    return not x == x


def main():
    if len(sys.argv) < 4:
        print("%s motif inmaf spec1,spec2,... " % sys.argv[0], file=sys.stderr)
        sys.exit(0)

    targmotif = sys.argv[1]
    inmaf = open(sys.argv[2])
    threshold = 0

    species = []

    for sp in sys.argv[3].split(','):
        species.append(sp)

    for maf in align_maf.Reader(inmaf):
        mafchrom = maf.components[0].src.split('.')[1]
        mafstart = maf.components[0].start
        mafend = maf.components[0].end
        reftext = maf.components[0].text

        # maf block scores for each matrix
        for scoremax, width, headers in MafMotifScorer(species, maf, targmotif):
            blocklength = width
            mafsrc, mafstart, mafend = headers[0]
            mafchrom = mafsrc.split('.')[1]

            # lists of scores for each position in scoremax
            mx = scoremax
            for offset in range(blocklength):

                # scan all species with threshold
                for i in range(len(species)):
                    if mx[i][offset] > threshold:
                        refstart = mafstart + offset - reftext.count('-', 0, offset)
                        refend = refstart + len(targmotif)
                        data = " ".join(["%.2f" % mx[x][offset] for x in range(len(species))])
                        # quote the motif
                        print(mafchrom, refstart, refend, "'"+targmotif+"'", data)
                        break


if __name__ == '__main__':
    main()



"""
Tests for `bx.seq.seq`.
"""

import unittest

import bx.seq
from bx.seq import fasta_tests, nib_tests, qdna_tests

test_fa = "test_data/seq_tests/test.fa"
test2_fa = "test_data/seq_tests/test2.fa"
test_nib = "test_data/seq_tests/test.nib"
test_qdna = "test_data/seq_tests/test.qdna"

valid_fasta = fasta_tests.valid_seq
valid_nib = nib_tests.valid_seq
valid_qdna = qdna_tests.valid_seq

# Same sequences as stored in test2.fa

valid2_fa = [("apple", "GGCGCTGCGATAAGGTTGCGACAACACGGACCTTCTTTTGCCTACCTCTGTTCTTGGCACG"),
             ("orange", "CGTGCCGAGAACAGAAAATACGCCGGGCGGTGCAGTAGTATCTTGGTATCCGATATGCAGG"),
             ("grapefruit", "CCTGCATATCGACTAGTACACCCTCCCGAGGTACCCCACCCATCCCTCTTTTCTCGGCGCG")]


class SEQTestCase (unittest.TestCase):

    def test_get_fasta(self):
        fastafile = bx.seq.seq_file(open(test_fa, "rb"))
        check_get(fastafile, valid_fasta, 3, 40)

    def test_get_nib(self):
        nibfile = bx.seq.seq_file(open(test_nib, "rb"))
        check_get(nibfile, valid_nib, 3, 40)

    def test_get_qdna(self):
        qdnafile = bx.seq.seq_file(open(test_qdna, "rb"))
        check_get(qdnafile, valid_qdna, 3, 40)

    def test_get_reader(self):
        reader = bx.seq.seq_reader(open(test2_fa, "rb"))
        for (ix, seq) in enumerate(reader):
            assert (ix < len(valid2_fa)), "FastaReader returns too many sequences"
            text = "%s" % seq
            fields = text.split()
            assert (len(fields) == 2), "SeqReader.__str__ returns incorrect sequence string \"%s\" (%d)" % text
            assert (fields[0] == valid2_fa[ix][0]), "FastaReader returned the wrong name (%s,%s)" % (fields[0], valid2_fa[ix][0])
            assert (fields[1] == valid2_fa[ix][1]), "FastaReader returned the wrong text (%s,%s)" % (fields[1], valid2_fa[ix][1])


def check_get(seqfile, valid_seq, start, len):
    assert seqfile.get(start, len) == valid_seq[start:start+len]



"""
Tools and data structures for working with genomic intervals (or sets of
regions on a line in general) efficiently.
"""

# For compatiblity with existing stuff
from bx.intervals.intersection import *  # noqa: F40



#!/usr/bin/env python

"""
Given two bed files print the number of bases covered 1) by both, 2) only by
the first, and 3) only by the second.

usage: %prog bed_file_1 bed_file_2
"""
from __future__ import print_function


from bx.bitset_builders import binned_bitsets_from_file
from bx.cookbook import doc_optparse


def coverage(bitsets):
    total = 0
    for chrom in bitsets:
        total += bitsets[chrom].count_range(0, bitsets[chrom].size)
    return total


options, args = doc_optparse.parse(__doc__)
try:
    in_fname, in2_fname = args
except ValueError:
    doc_optparse.exit()

bits1 = binned_bitsets_from_file(open(in_fname))
bits2 = binned_bitsets_from_file(open(in2_fname))

bits1_covered = coverage(bits1)
bits2_covered = coverage(bits2)

bitsets = dict()

for key in bits1:
    if key in bits2:
        bits1[key].iand(bits2[key])
        bitsets[key] = bits1[key]

both_covered = coverage(bitsets)

print("in both:  \t%d" % both_covered)
print("only in %s:\t%d" % (in_fname, bits1_covered - both_covered))
print("only in %s:\t%d" % (in2_fname, bits2_covered - both_covered))



"""
Tests for `bx.seq.nib`.
"""

import unittest

from . import nib

test_nib = "test_data/seq_tests/test.nib"

# Same sequence data as stored in test.nib

valid_seq = "TGGAGGCATTTGTGATTCAATAGATGCAGAAAGAAACCTTCCTAGAGCTG" \
    + "GCGTTCTCTAACTAAAAGTGGAAAGTTCTGAGGAATGAGGACTGTTATAA" \
    + "ATCCCACCCCACACCGCACCTTCTCCAGGGAAGTTTCATGGCCGTGAAGA" \
    + "GGACAGAAAGTGAGAACCAAGATggaactgaataaacaagcttcacactg" \
    + "ttagtttccccatatgcttaccttcccacagatgccaaccttggaggcct" \
    + "aagaggcctagaatattatcctttgtctgatcatttctctacaaatttat" \
    + "tgttctttgttaagatgctacataagcccaaattctaaccacccctttga" \
    + "gttacccatcatcaagtttctcccatgtg"

valid_seq_len = len(valid_seq)


class NIBTestCase(unittest.TestCase):

    def test_get(self):
        nibfile = nib.NibFile(open(test_nib, 'rb'))
        # Try all combinations of even / odd boundaries
        check_get(nibfile, 0, 10)
        check_get(nibfile, 1, 10)
        check_get(nibfile, 0, 11)
        check_get(nibfile, 1, 11)
        # Test near end of file also
        check_get(nibfile, valid_seq_len - 10, 10)
        check_get(nibfile, valid_seq_len - 11, 11)
        # Test really short gets
        check_get(nibfile, 0, 0)
        check_get(nibfile, 1, 0)
        check_get(nibfile, 0, 1)
        check_get(nibfile, 1, 1)
        # Test negative length
        self.assertRaises(AssertionError, nibfile.get, 20, -1)


def check_get(nibfile, start, len):
    assert nibfile.get(start, len) == valid_seq[start:start+len]



"""
Support for masking out sites in alignments based on sequence quality. Both
simple masking of regions below some threshold and masking using the
neighborhood quality standard (NQS) are supported. Uses sequence quality
values stored in a `bx.binned_array.FileBinnedArray`.
"""

from bx.align.sitemask import Masker
from bx.binned_array import FileBinnedArray


# This class implements simple rules for masking quality, if base <
# minqual, mask
class Simple(Masker):
    # keys should be:
    # qualspecies: dictionary of species as key, lengths
    #              dict by chromosome or chromosome list as value
    # qualfiles: prefix for quality file for each species in qualspecies
    # mask: mask character (default is '?')
    # minqual: minimum quality
    # cache: optional, but sets the number of megabytes allowed in cache per quality masked species
    def __init__(self, qualfiles=None, qualspecies=None, minqual=None, mask="?", cache=100):
        if not qualfiles:
            raise Exception("No quality files.")
        if not qualspecies:
            raise Exception("No species dictionary.")
        if not minqual:
            raise Exception("No minimum quality specified.")
        self.mask = "?"
        self.minqual = minqual
        self.mask = mask
        self.total = 0
        self.masked = 0

        self.qualfiles = qualfiles
        self.qualspecies = qualspecies
        self.cache = cache * 2  # typical bin size is 512K
        # load quality files into FileBinnedArray
        self.qualities = {}
        for species, qualfile in self.qualfiles.items():
            specdict = {}
            for chrom in self.qualspecies[species]:
                specdict[chrom] = FileBinnedArray(
                    open(qualfile + "." + chrom + ".bqv", "rb"),
                    cache=self.cache/len(qualfiles))
            self.qualities[species] = specdict

    def __call__(self, block):
        if not block:
            return
        for qualspec in self.qualities:
            comp = block.get_component_by_src_start(qualspec)
            if not comp:
                continue
            chrom = comp.src.split(".")[1]
            start, end = comp.get_forward_strand_start(), comp.get_forward_strand_end()
            # get quality slice, for + strand
            qual = self.qualities[qualspec][chrom][start:end]
            x = 0
            while start+x < end:
                self.total += 1
                # got the column in the alignment for this particular base
                if qual[x] < self.minqual:
                    col = comp.coord_to_col(start+x)
                    self.masked += 1
                    for component in block.components:
                        if component.text[col] != "-":
                            component.text = component.text[0:col] + \
                                self.mask + \
                                component.text[col+1:len(component.text)]
                # iterate through quality
                x += 1
        return block


class NQS(Masker):
    # keys should be:
    # qualspecies: dictionary of species as key, lengths
    #              dict by chromosome or chromosome list as value
    # qualfiles: prefix for quality file for each species in qualspecies
    # mask: mask character (default is '?')
    # minqual: minimum quality
    # neighborqual: neighborhood minimum quality (bases within 5 bps are masked)
    # cache: optional, but sets the number of megabytes allowed in cache per quality masked species
    def __init__(self, qualfiles=None, qualspecies=None, minqual=None, mask="?", cache=100):
        if not qualfiles:
            raise Exception("No quality files.")
        if not qualspecies:
            raise Exception("No species dictionary.")
        if not minqual:
            raise Exception("No minimum quality specified.")
        self.mask = "?"
        self.minqual = minqual
        self.mask = mask
        self.total = 0
        self.masked = 0

        self.qualfiles = qualfiles
        self.qualspecies = qualspecies
        self.cache = cache * 2  # typical bin size is 512K
        # load quality files into FileBinnedArray
        self.qualities = {}
        for species, qualfile in self.qualfiles.items():
            specdict = {}
            for chrom in self.qualspecies[species]:
                specdict[chrom] = FileBinnedArray(
                    open(qualfile + "." + chrom + ".bqv", "rb"),
                    cache=self.cache/len(qualfiles))
            self.qualities[species] = specdict

    def __call__(self, block):
        if not block:
            return
        for qualspec in self.qualities:
            comp = block.get_component_by_src_start(qualspec)
            chrom = comp.src.split(".")[1]
            start, end = comp.get_forward_strand_start(), comp.get_forward_strand_end()
            # get quality slice, for + strand
            qual = self.qualities[qualspec][chrom][start:end]
            x = 0
            while start+x < end:
                self.total += 1
                # got the column in the alignment for this particular base
                if qual[x] < self.minqual:
                    col = comp.coord_to_col(start+x)
                    self.masked += 1
                    for component in block.components:
                        if component.text[col] != "-":
                            component.text = component.text[0:col] + \
                                self.mask + \
                                component.text[col+1:len(component.text)]
                # iterate through quality
                x += 1
        return block



#!/usr/bin/env python

"""
Merge any overlapping regions of bed files. Bed files can be provided on the
command line or on stdin. Merged regions are always reported on the '+'
strand, and any fields beyond chrom/start/stop are lost.

usage: %prog bed files ...
"""
from __future__ import print_function

import fileinput
import sys

from bx.bitset_builders import binned_bitsets_from_bed_file

bed_filenames = sys.argv[1:]
if bed_filenames:
    input = fileinput.input(bed_filenames)
else:
    input = sys.stdin

bitsets = binned_bitsets_from_bed_file(input)

for chrom in bitsets:
    bits = bitsets[chrom]
    end = 0
    while True:
        start = bits.next_set(end)
        if start == bits.size:
            break
        end = bits.next_clear(start)
        print("%s\t%d\t%d" % (chrom, start, end))



from __future__ import print_function

import os
import sys
import unittest
from functools import partial

import numpy

try:
    sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
except Exception:
    sys.path.insert(0, os.path.dirname(os.path.abspath(".")))

from bx.bbi.bigwig_file import BigWigFile


def allclose(a, b, tol=0.00001):
    """
    Like numpy.allclose but treat Nan == Nan
    """
    d = numpy.absolute(a - b)
    return numpy.all(numpy.isnan(d) | (d < tol))


class TestBigWig(unittest.TestCase):
    def setUp(self):
        f = open("test_data/bbi_tests/test.bw", 'rb')
        self.bw = BigWigFile(file=f)

    def test_get_summary(self):
        data = self.bw.query("chr1", 10000, 20000, 10)
        means = [x['mean'] for x in data]
        print(means)
        assert numpy.allclose([float(_) for _ in means], [-0.17557571594973645, -0.054009292602539061, -0.056892242431640622, -0.03650328826904297, 0.036112907409667966, 0.0064466032981872557, 0.036949024200439454, 0.076638259887695306, 0.043518108367919923, 0.01554749584197998])

        # Summarize variant
        sd = self.bw.summarize("chr1", 10000, 20000, 10)
        assert numpy.allclose(sd.sum_data / sd.valid_count, [-0.17557571594973645, -0.054009292602539061, -0.056892242431640622, -0.03650328826904297, 0.036112907409667966, 0.0064466032981872557, 0.036949024200439454, 0.076638259887695306, 0.043518108367919923, 0.01554749584197998])

        # Test min and max for this entire summary region
        data = self.bw.query("chr1", 10000, 20000, 1)
        maxs = [x['max'] for x in data]
        mins = [x['min'] for x in data]
        self.assertEqual([float(_) for _ in maxs], [0.289000004529953])
        self.assertEqual([float(_) for _ in mins], [-3.9100000858306885])

    def test_get_leaf(self):
        data = self.bw.query("chr1", 11000, 11005, 5)
        means = [x['mean'] for x in data]
        assert numpy.allclose([float(_) for _ in means], [0.050842501223087311, -2.4589500427246094, 0.050842501223087311, 0.050842501223087311, 0.050842501223087311])

        # Test min and max for this entire leaf region
        data = self.bw.query("chr1", 11000, 11005, 1)
        maxs = [x['max'] for x in data]
        mins = [x['min'] for x in data]
        self.assertEqual([float(_) for _ in maxs], [0.050842501223087311])
        self.assertEqual([float(_) for _ in mins], [-2.4589500427246094])

    def test_wrong_nochrom(self):
        data = self.bw.query("chr2", 0, 10000, 10)
        self.assertEqual(data, None)

# Nose test generator


def test_summaries_from_file():
    bw = BigWigFile(file=open("test_data/bbi_tests/test.bw", 'rb'))

    def check_summary(line):
        fields = line.split()
        chrom = fields[0]
        start = int(fields[1])
        end = int(fields[2])
        n = int(fields[3])
        t = fields[4]
        values = [float(v.replace('n/a', 'NaN')) for v in fields[5:]]
        sd = bw.summarize(chrom, start, end, n)
        if t == 'mean':
            print(sd.sum_data / sd.valid_count)
            print(values)
            assert allclose(sd.sum_data / sd.valid_count, values)
        elif t == 'min':
            assert allclose(sd.min_val, values)
        elif t == 'max':
            assert allclose(sd.max_val, values)
        # elif t == 'std':
        #    assert numpy.allclose( sd.max_val, values )
    for i, line in enumerate(open("test_data/bbi_tests/test.expectation")):
        f = partial(check_summary, line)
        f.description = "Test summaries line %d: %s" % (i, line[:40])
        yield (f, )


if __name__ == '__main__':
    unittest.main()



"""
Various utilities for working with `bx.align.Alignment` objects.
"""

from .chop import *  # noqa: F40
from .fuse import *  # noqa: F40
from .thread import *  # noqa: F40
from .tile import *  # noqa: F40



"""
Provides functions for creating simple properties.

If, inside a class definition, you write:

    attribute(foo=1, bar=2)

simple properties named 'foo' and 'bar' are created for this class.
Also, private instance variables '__foo' and '__bar' will be added
to instances of this class.

USEAGE:

# assumes attribute.py is on path
from attribute import *

class MyClass(object):
    readable(foo=1, bar=2) # or, attribute('r', foo=1, bar=2)
    writable(fro=3, boz=4) # or, attribute('w', fro=3, boz=4)
    attribute(baz=5)

This is equivalent to the following:

class MyClass(object):
    def __init__(self):
        self.__foo = 1
        self.__bar = 2
        self.__fro = 3
        self.__boz = 4
        self.__baz = 5

    def get_foo(self):
        return self.__foo
    def get_bar(self):
        return self.__bar
    def set_fro(self, value):
        self.__fro = value
    def set_boz(self, value):
        self.__boz = value
    def get_baz(self):
        return self.__baz
    def set_baz(self, value):
        self.__baz = value
    def del_baz(self):
        del self.__baz

    foo = property(fget=get_foo, doc="foo")
    bar = property(fget=get_bar, doc="bar")
    fro = property(fset=set_fro, doc="fro")
    boz = property(fset=set_boz, doc="boz")
    baz = property(fget=get_baz, fset=set_baz, fdel=del_baz, doc="baz")
"""

__all__ = ['attribute', 'readable', 'writable']
__version__ = '3.0'
__author__ = 'Sean Ross'
__credits__ = ['Guido van Rossum', 'Garth Kidd']
__created__ = '10/21/02'

import sys


def mangle(classname, attrname):
    """mangles name according to python name-mangling
       conventions for private variables"""
    return "_%s__%s" % (classname, attrname)


def class_space(classlevel=3):
    "returns the calling class' name and dictionary"
    frame = sys._getframe(classlevel)
    classname = frame.f_code.co_name
    classdict = frame.f_locals
    return classname, classdict

# convenience function


def readable(**kwds):
    "returns one read-only property for each (key,value) pair in kwds"
    return _attribute(permission='r', **kwds)

# convenience function


def writable(**kwds):
    "returns one write-only property for each (key,value) pair in kwds"
    return _attribute(permission='w', **kwds)

# needed because of the way class_space is resolved in _attribute


def attribute(permission='rwd', **kwds):
    """returns one property for each (key,value) pair in kwds;
       each property provides the specified level of access(permission):
           'r': readable, 'w':writable, 'd':deletable
    """
    return _attribute(permission, **kwds)

# based on code by Guido van Rossum, comp.lang.python 2001-07-31


def _attribute(permission='rwd', **kwds):
    """returns one property for each (key,value) pair in kwds;
       each property provides the specified level of access(permission):
           'r': readable, 'w':writable, 'd':deletable
    """
    classname, classdict = class_space()

    def _property(attrname, default):
        propname, attrname = attrname, mangle(classname, attrname)
        fget, fset, fdel, doc = None, None, None, propname
        if 'r' in permission:
            def fget(self):
                value = default
                try:
                    value = getattr(self, attrname)
                except AttributeError:
                    setattr(self, attrname, default)
                return value
        if 'w' in permission:
            def fset(self, value):
                setattr(self, attrname, value)
        if 'd' in permission:
            def fdel(self):
                try:
                    delattr(self, attrname)
                except AttributeError:
                    pass
                # calling fget can restore this attribute, so remove property
                delattr(self.__class__, propname)
        return property(fget=fget, fset=fset, fdel=fdel, doc=doc)

    for attrname, default in kwds.items():
        classdict[attrname] = _property(attrname, default)



"""
Classes to support nib files.

:Author: James Taylor (james@bx.psu.edu), Bob Harris (rsharris@bx.psu.edu)

A nib sequence is a sequence of DNA, using the 10 character alphabet A,C,G,T,N
(upper and lower case).  The file is packed as 4 bits per character.

nib file format
---------------

Fields can be in big- or little-endian format;  they must match the endianess
of the magic number.

============ =========== ======================================================
offset 0x00: 6B E9 3D 3A big endian magic number (3A 3D E9 6B => little endian)
offset 0x04: xx xx xx xx length of data sequence (counted in characters)
offset 0x08:  ...        data sequence;  most significant nybble in each
                         byte is first in sequence
============ =========== ======================================================
"""
from __future__ import division

import math
import struct

from bx.seq.seq import SeqFile, SeqReader
from . import _nib

NIB_MAGIC_NUMBER = 0x6BE93D3A
NIB_MAGIC_NUMBER_SWAP = 0x3A3DE96B
NIB_MAGIC_SIZE = 4
NIB_LENGTH_SIZE = 4


class NibFile(SeqFile):

    def __init__(self, file, revcomp=False, name="", gap=None):
        SeqFile.__init__(self, file, revcomp, name, gap)

        self.byte_order = ">"
        magic = struct.unpack(">L", file.read(NIB_MAGIC_SIZE))[0]
        if (magic != NIB_MAGIC_NUMBER):
            if magic == NIB_MAGIC_NUMBER_SWAP:
                self.byte_order = "<"
            else:
                raise Exception("Not a NIB file")
        self.magic = magic
        self.length = struct.unpack("%sL" % self.byte_order, file.read(NIB_LENGTH_SIZE))[0]

    def raw_fetch(self, start, length):
        # Check parameters
        assert start >= 0, "Start must be greater than 0"
        assert length >= 0, "Length must be greater than 0"
        assert start + length <= self.length, "Interval beyond end of sequence"
        # Read block of bytes containing sequence
        block_start = int(math.floor(start / 2))
        block_end = int(math.floor((start + length - 1) / 2))
        block_len = block_end + 1 - block_start
        self.file.seek(NIB_MAGIC_SIZE + NIB_LENGTH_SIZE + block_start)
        raw = self.file.read(block_len)
        # Unpack compressed block into a character string and return
        return _nib.translate_raw_data(raw, start, length)


class NibReader(SeqReader):

    def __init__(self, file, revcomp=False, name="", gap=None):
        SeqReader.__init__(self, file, revcomp, name, gap)

    def __next__(self):
        if (self.seqs_read != 0):
            return  # nib files have just one sequence
        seq = NibFile(self.file, self.revcomp, self.name, self.gap)
        self.seqs_read += 1
        return seq


class NibWriter(object):

    def __init__(self, file):
        self.file = file

    def write(self, seq):
        assert (False), "NibWriter.write() is not implemented yet"

    def close(self):
        self.file.close()



#!/usr/bin/env python

"""
Read a file containing a 0 or 1 on each line (`feature_file`), output
all lines from stdin for which that value was 1

TODO: no need to read the feature_file into memory here, just iterate in
      parallel.

usage: %prog feature_file < ...
"""
from __future__ import print_function

import sys


def __main__():

    feature_file = sys.argv[1]

    if len(sys.argv) > 2:
        match = int(sys.argv[2])
    else:
        match = 1

    feature_vector = [int(line) for line in open(feature_file)]

    for index, line in enumerate(sys.stdin):
        if feature_vector[index] == match:
            print(line, end='')


if __name__ == "__main__":
    __main__()



"tests for bx.align.epo"

import pdb
import random
import unittest

import numpy as np

from bx.align._epo import (
    bed_union,
    cummulative_intervals,
)
from bx.align.epo import (
    Chain,
    EPOitem
)


class TestBed(unittest.TestCase):
    def setUp(self):
        self.N = random.randint(1, 1000)

    def test_ci(self):
        S, D = [], []
        for i in range(self.N):
            S.append(random.randint(10, 50))
            D.append(random.randint(10, 50))
        D[-1] = 0
        C = cummulative_intervals(np.array(S, dtype=np.int64), np.array(D, dtype=np.int64))
        for i in range(self.N):
            assert C[i, 1] - C[i, 0] == S[i]
        for i in range(1, self.N):
            assert C[i, 0] - C[i-1, 1] == D[i-1], "[%d] %d != %d" % (i, C[i, 0] - C[i-1, 1], D[i-1])

    def test_elem_u(self):
        # back to back, so should return a single interval
        EL = []
        th = 0
        for i in range(self.N):
            size = random.randint(1, 20)
            EL.append((th, th+size))
            th += size
        U = bed_union(np.array(EL, dtype=np.uint64))
        assert U[0, 0] == 0 and U[0, 1] == th

        # disjoint
        EL = []
        th = 0
        for i in range(self.N):
            size = random.randint(1, 20)
            EL.append((th, th+size))
            th += (size + 1)
        U = bed_union(np.array(EL, dtype=np.uint64))
        for i in range(U.shape[0]):
            assert (U[i, 0], U[i, 1]) == EL[i]

        # random with some empty elements
        EL = []
        th = 0
        for i in range(self.N):
            size = random.randint(1, 20)
            EL.append((th, th+size))
            th += random.randint(1, size+size)  # 50% of overlapping
        U = bed_union(np.array(EL, dtype=np.uint64))

        assert U[0, 1] > U[0, 0]
        for i in range(1, U.shape[0]):
            assert U[i, 1] > U[i, 0]
            assert U[i, 0] > U[i-1, 1]


cigar_pairs = [
    ("GGACCTGGAGAGATCAG---------------------------GACTTCAACTGTGTG-------------TCTTAGACTGGG--------AGGGTGTTA",
     "AGGCCAGGAGAGATCAGGTAAGTCTTAATTTAATAAAGAGATAGGACCTGAACTGTGTCTAACAATAGGTAATATTAGACTGGGGGAGAGAGAAGACTTTC"),
    ("TTT--------------------------------------------------------------------------------------------------------------------T",
     "CTTGTACCAAGGACAGTACTGGCAGCCTAATTGCTAACACTTTGTGGTGGATTGGTCCACTCAATATTTGTTCCCACCTCTTTTCAGTCCAGTTCTATAAAGGACAGAAAGTTGAAAACT"),
    ("A-------------------------------------------------ACACTGGACACAGCACTAACACGATTACTTA",
     "ACATTTCCCACACTCCCTTGCAGCTAGGTTTCTAGATATAATTTAGATTCCA----------------------------A"),
    ("TTTGGTCCTCTGGA------CGAGCAGCCAGTGCT---------------------------------------------------------------------------AAAAAAAA",
     "T---CATTCTAGCAGGTGCTGCAGCAGCAGGTAGCCCTGGAGCCAACAGTTGTGGCTATGATTCTTGATCATCAGATTTGGCTCAAGTGATGTGTTCCTCTAGCATGCACTTGAGATA"),
    ("G-----------------------C----------------------------------------------------------------------------------------A",
     "GGCCTGCACTGCCAGTAATTTTAACAAATTTTTAGGCACTGAATTCCCTGTATTAAATCTGTTTTCCTTAGCGTAAACAGATCTCTGTTAAATGAAACTAAACCCTGACTGATA"),
    ("TATT----------------------------------T",
     "TCCTTCATTTTATTTCTCCCTTAAAATTTTTTTTATTACT"),
    ("TAAAAA--A------A------------------------------------------------------------TTTTTTTTTTT",
     "T---AATTATTTTGCAGCAGGTCCTTGATAACATATCATCTATAAATATTTCAGCAAGAATCTCTAAAAGGCAAGAACCTCCTTCTT"),
    ("AAACAA---------------------------------------TT---T",
     "AAACAATACCACTGCATCACTATCAAACCCAAAAAATAACAAAAATTGGGT"),
    ("TCTTAAC---TGCTGAGCCATCCCTCCAGCTCCTGTTTTATTTTTATTATGAAGTAATAATA--ATAG--TAATAATAATGATG",
     "TACACTTAATTCTAAAACTTGTTATGAATCATCA----------TTGG--TTTTTTATTGTGAAGAACTAATATAATCAGA--G"),
    ("ATGATAATGGTATCCTAGCTCAACACCTG-GAGTTCACCCCAACAGTTAACTAA----GTTTGAGGAAGTGTTAACAAGCCTA---ACAAAGAGGACATGCCAATAGCTGACAGAGTCAC",
     "A-------CCTCTGCTAGCTCAACTCCTGAGAATCAATTATATAAGCTAGGTCAGTGGTTTTGAGAAAGTATTAGTAGACATTTCTCCAAAGAATACATAAAAATGGCC-A--CAAGTAT")
]


def toCigar(species, id, s):
    I = [(0, 0)]
    L = [len(_) for _ in s.split("-")]
    NZ = [_ for _ in L if _]

    if L[0] > 0:
        I.append((0, L[0]))
        NZ = NZ[1:]
        L = L[1:]

    for i in range(len(NZ)):
        L.insert(0, 0)
        size = NZ[i]
        start = L.index(size)
        I.append((I[-1][1] + start, I[-1][1]+start+size))
        L = L[start+1:]
    if len(L):
        I.append((I[-1][1] + len(L), I[-1][1] + len(L)))
    C = []
    for i in range(1, len(I)):
        dl = I[i][0] - I[i-1][1]
        ml = I[i][1] - I[i][0]

        dc = ""
        if dl:
            dc = (dl > 1 and str(dl) or "") + "D"

        mc = ""
        if ml:
            mc = (ml > 1 and str(ml) or "") + "M"

        C.append(dc+mc)
    MSUM = sum(i[1]-i[0] for i in I)
    start = random.randint(50, 10000)
    return "%s\t%d\t1\t%d\t%d\t%d\t%s" % (species, id, start, start+MSUM-1, random.choice((-1, 1)), "".join(C))


class TestEpo(unittest.TestCase):
    def setUp(self):
        self.epo_records = []
        for i, (t, q) in enumerate(cigar_pairs):
            gab_pair = (toCigar("homo_sapiens", i, t), toCigar("mus_musculus", i, q))
            A = EPOitem._strfactory(gab_pair[0])
            B = EPOitem._strfactory(gab_pair[1])
            if A and B:
                self.epo_records.append((A, B))

    def test_out(self):
        def ch(c, ci):
            th = 0
            for l, t in ci:
                if t == 'M':
                    assert c[th:th+l].find('-') == -1
                else:
                    assert c[th:th+l] == '-' * l
                th += l

        for (a, b) in self.epo_records:
            ca, cb = cigar_pairs[int(a.gabid)]
            ch(ca, a.cigar_iter(False))
            ch(cb, b.cigar_iter(False))

    def test_make_chain(self):
        def cch(cigar, s, e):
            return cigar[s:e].find('-') == -1

        for p in self.epo_records:
            chain = Chain._make_from_epo(p[0], p[1], {"chr1": 500}, {"chr1": 800})
            if not chain:
                continue
            ch, S, T, Q = chain
            i = int(ch.id)
            c1, c2 = cigar_pairs[i]
            if p[0].strand == '-':
                c1 = c1[::-1]
                c2 = c2[::-1]
            th = 0
            for s, t, q in zip(S, T, Q):
                if not (cch(c1, th, th+s) and cch(c2, th, th+s)):
                    pdb.set_trace()
                assert cch(c1, th, th+s) and cch(c2, th, th+s), "%s and %s" % (c1[th:th+s], c2[th:th+s])
                if t > q:
                    cch(c1, th+s, th+s+t) and c1[th+s:th+s+t] == '-'*t
                else:
                    cch(c2, th+s, th+s+q) and c1[th+s:th+s+q] == '-'*q
                th = th + s + max(t, q)

    def test_rem_dash(self):
        # ****--****-------****  4M2D4M7D4M
        # *******-------*******  7M7D7M
        # has 4 dash columns and should become
        # ****--****---****      4M2D4M3D4M
        # *******---*******      7M3D7M

        for i in range(100):
            dash_cols = random.randint(0, 10)
            tStart = random.randint(0, 1000)
            qStart = random.randint(0, 1000)
            epo_pair = (
                EPOitem._strfactory("homo_sapiens\t0\t1\t%d\t%d\t1\t%s" % (tStart, tStart+12-1, "4M2D4M%dD4M" % (dash_cols+3))),
                EPOitem._strfactory("mus_musculus\t0\t1\t%d\t%d\t1\t%s" % (qStart, qStart+14-1, "7M%dD7M" % (dash_cols+3))))
            chain = Chain._make_from_epo(epo_pair[0], epo_pair[1], {"chr1": 500}, {"chr1": 800})
            ti = epo_pair[0].intervals(False)
            qi = epo_pair[1].intervals(False)
            assert ti[2][0] - ti[1][1] - dash_cols == chain[2][1]
            assert qi[1][0] - qi[0][1] - dash_cols == chain[2][1]

        # ----*****
        # *-------*
        # has 3 dash cols and should become
        # *
        # *
        # with the qStart += 1 and tStart += 4

        for i in range(100):
            dash_cols = random.randint(0, 10)
            tm = random.randint(6, 10)
            qm = random.randint(1, 5)

            tStart = random.randint(0, 1000)
            qStart = random.randint(0, 1000)

            epo_pair = (
                EPOitem._strfactory("homo_sapiens\t0\t1\t%d\t%d\t1\t%s" % (tStart, tStart+tm-1, "%dD%dM" % (dash_cols+1, tm))),
                EPOitem._strfactory("mus_musculus\t0\t1\t%d\t%d\t1\t%s" % (qStart, qStart+qm+1-1, "M%dD%dM" % (dash_cols+tm-qm, qm))))
            chain = Chain._make_from_epo(epo_pair[0], epo_pair[1], {"chr1": 500}, {"chr1": 800})
            if chain[1][-1] != qm:
                pdb.set_trace()
            assert chain[1][-1] == qm
            # correct also for coordinate interpretation differences between UCSC and EPO
            assert (qStart + 1) - 1 == chain[0].qStart, "%d != %d" % (qStart + 1, chain[0].qStart)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Subtract one set of genomic intervals from another (base-by-base or whole
intervals). The returned GenomicIntervals will be in the order
of the first set of intervals passed in, with the corresponding
meta-data.
"""

from warnings import warn

from bx.intervals.io import (
    BitsetSafeReaderWrapper,
    GenomicInterval
)
from bx.intervals.operations import bits_clear_in_range
from bx.tabular.io import (
    Comment,
    Header,
)


def subtract(readers, mincols=1, upstream_pad=0, downstream_pad=0, pieces=True, lens={}, comments=True):
    # The incoming lens dictionary is a dictionary of chromosome lengths which are used to initialize the bitsets.
    # Read all but first into bitsets and union to one (if confused, read DeMorgan's...)
    primary = readers[0]
    union = readers[1:]
    # Handle any ValueError, IndexError and OverflowError exceptions that may be thrown when
    # the bitsets are being created by skipping the problem lines
    union[0] = BitsetSafeReaderWrapper(union[0], lens=lens)
    bitsets = union[0].binned_bitsets(upstream_pad=upstream_pad, downstream_pad=downstream_pad, lens=lens)
    union = union[1:]
    for andset in union:
        bitset2 = andset.binned_bitsets(upstream_pad=upstream_pad, downstream_pad=downstream_pad, lens=lens)
        for chrom in bitset2:
            if chrom not in bitsets:
                bitsets[chrom] = bitset2[chrom]
            else:
                bitsets[chrom].ior(bitset2[chrom])

    # Read remaining intervals and subtract
    for interval in primary:
        if isinstance(interval, Header):
            yield interval
        if isinstance(interval, Comment) and comments:
            yield interval
        elif isinstance(interval, GenomicInterval):
            chrom = interval.chrom
            if chrom not in bitsets:
                yield interval
            else:
                start = int(interval.start)
                end = int(interval.end)
                if start > end:
                    warn("Interval start after end!")
                out_intervals = []
                # Find the intervals that meet the criteria (for the three sensible
                # permutations of reverse and pieces)
                try:
                    if bitsets[chrom].count_range(start, end-start) >= mincols:
                        if pieces:
                            out_intervals = bits_clear_in_range(bitsets[chrom], start, end)
                    else:
                        out_intervals = [(start, end)]
                    # Write the intervals
                    for start, end in out_intervals:
                        new_interval = interval.copy()
                        new_interval.start = start
                        new_interval.end = end
                        yield new_interval
                except IndexError as e:
                    try:
                        # This will work only if primary is a NiceReaderWrapper
                        primary.skipped += 1
                        # no reason to stuff an entire bad file into memmory
                        if primary.skipped < 10:
                            primary.skipped_lines.append((primary.linenum, primary.current_line, str(e)))
                    except Exception:
                        pass
                    continue



"""
Reading and writing delimited data files (with headers and comments).
"""
from itertools import count

from six import Iterator

FIRST_LINE_IS_HEADER = object()


class ParseError(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args)
        self.linenum = kwargs.get("linenum", None)

    def __str__(self):
        if self.linenum:
            return Exception.__str__(self) + " on line " + str(self.linenum)
        else:
            return Exception.__str__(self)


class TableRow(object):
    """
    A row of a table
    """

    def __init__(self, reader, fields):
        self.reader = reader
        self.fields = fields

    def __getitem__(self, key):
        if isinstance(key, int):
            return self.fields[key]
        elif isinstance(key, str):
            if self.reader.header:
                return self.fields[self.reader.header.field_to_column[key]]
            else:
                raise TypeError("column names only supported for files with headers")
        else:
            raise TypeError("field indices must be integers or strings")

    @property
    def fieldnames(self):
        return self.reader.header.fields

    def __str__(self):
        return "\t".join(self.fields)


class Header(object):
    """
    Header of a table -- contains column names and a mapping from them
    to column indexes
    """

    def __init__(self, fields):
        self.set_fields(fields)

    def set_fields(self, fields):
        self.fields = fields
        self.field_to_column = dict(zip(fields, count()))

    def __getitem__(self, key):
        if isinstance(key, int):
            return self.fields[key]
        elif isinstance(key, str):
            if key in self.field_to_column:
                return key
        else:
            raise TypeError("field indices must be integers or strings")

    def __str__(self):
        return "#" + "\t".join(self.fields)


class Comment(object):
    def __init__(self, line):
        self.line = line

    def __str__(self):
        if self.line.startswith("#"):
            return self.line
        return "#" + self.line


class TableReader(Iterator):
    """
    Reader for iterating tabular data
    """

    def __init__(self, input, return_header=True, return_comments=True, force_header=None, comment_lines_startswith=["#"]):
        self.input = input
        self.return_comments = return_comments
        self.return_header = return_header
        self.input_iter = iter(input)
        self.linenum = 0
        self.header = force_header
        self.comment_lines_startswith = comment_lines_startswith

    def __iter__(self):
        return self

    def __next__(self):
        line = next(self.input_iter)
        self.linenum += 1
        line = line.rstrip("\r\n")
        # Catch blank lines (throw a warning?)
        # This will end up adding a '#' at the beginning of blank lines
        if line == '':
            if self.return_comments:
                return Comment(line)
            else:
                return next(self)
        # Force header?
        if self.header is FIRST_LINE_IS_HEADER and self.linenum == 1:
            self.header = self.parse_header(line)
            if self.return_header:
                return self.header
            else:
                return next(self)
        # Is it a comment line?
        for comment_line_start in self.comment_lines_startswith:
            if line.startswith(comment_line_start):
                # If a comment and the first line we assume it is a header
                if self.header is None and self.linenum == 1:
                    self.header = self.parse_header(line)
                    if self.return_header:
                        return self.header
                    else:
                        return next(self)
                else:
                    if self.return_comments:
                        return self.parse_comment(line)
                    else:
                        return next(self)
        # Not a comment, must be an interval
        try:
            return self.parse_row(line)
        except ParseError as e:
            e.linenum = self.linenum
            raise e

    def parse_header(self, line):
        if line.startswith("#"):
            fields = line[1:].split("\t")
        else:
            fields = line.split("\t")
        return Header(fields)

    def parse_comment(self, line):
        return Comment(line)

    def parse_row(self, line):
        return TableRow(self, line.split("\t"))



"""
Pyrex extension to speed up some operations in `core.py`.
"""

def coord_to_col( int start, char * text, int pos ):
    cdef int col
    col = 0
    while start < pos:
        # Note: ord( '-' ) = 45
        if text[col] != 45: 
            start = start + 1
        col = col + 1 
    return col


"""
Complement a set of intervals.
"""

from bx.bitset import MAX
from bx.intervals.io import (
    BitsetSafeReaderWrapper,
    GenomicInterval
)
from bx.intervals.operations import bits_set_in_range


def complement(reader, lens):
    # Handle any ValueError, IndexError and OverflowError exceptions that may be thrown when
    # the bitsets are being created by skipping the problem lines
    complement_reader = BitsetSafeReaderWrapper(reader, lens=lens)
    bitsets = complement_reader.binned_bitsets(upstream_pad=0, downstream_pad=0, lens=lens)
    # NOT them all
    for key, value in bitsets.items():
        value.invert()
    # Read remaining intervals and subtract
    for chrom in bitsets:
        bitset = bitsets[chrom]
        out_intervals = bits_set_in_range(bitset, 0, lens.get(chrom, MAX))
        try:
            # Write the intervals
            for start, end in out_intervals:
                fields = ["." for x in range(max(complement_reader.chrom_col, complement_reader.start_col, complement_reader.end_col)+1)]
                # default the column to a + if it exists
                if complement_reader.strand_col < len(fields) and complement_reader.strand_col >= 0:
                    fields[complement_reader.strand_col] = "+"
                fields[complement_reader.chrom_col] = chrom
                fields[complement_reader.start_col] = start
                fields[complement_reader.end_col] = end
                new_interval = GenomicInterval(complement_reader, fields, complement_reader.chrom_col, complement_reader.start_col, complement_reader.end_col, complement_reader.strand_col, "+")
                yield new_interval
        except IndexError as e:
            complement_reader.skipped += 1
            # no reason to stuff an entire bad file into memmory
            if complement_reader.skipped < 10:
                complement_reader.skipped_lines.append((complement_reader.linenum, complement_reader.current_line, str(e)))
            continue


# def main():
#     # test it all out
#     f1 = fileinput.FileInput("dataset_7.dat")
#     g1 = GenomicIntervalReader(f1)
#     for interval in complement(g1,{"chr":16000000}):
#         print "\t".join(interval)
#
# if __name__ == "__main__":
#     main()



"""
Iterator for the BED format ( http://genome.ucsc.edu/FAQ/FAQformat.html#format1 )
Returns chrom, chromStart, chromEnd, name, score
"""

cdef class BedReader:
    cdef object f
    def __init__( self, f ):
        self.f = f

    def __iter__( self ):
        return self

    def __next__( self ):
        while True:
            line = self.f.readline()
            if not line:
                raise StopIteration()
            if line.isspace():
                continue    
            if line[0] == "#":
                continue
            if line[0].isalpha():
                if line.startswith( "track" ) or line.startswith( "browser" ):
                    continue
                
                feature = line.strip().split()
                chrom = feature[0]
                chrom_start = int(feature[1])
                chrom_end = int(feature[2])
                if len(feature) > 3:
                    name = feature[3]
                else:
                    name = None
                                
                if len(feature) > 4:
                    score = int(feature[4])
                else:
                    score = None
                    
                
                return chrom, chrom_start, chrom_end, name, score
                    
            else:
                raise "Unexpected input line: %s" % line.strip()




#!/usr/bin/env python

"""
Pass through blocks from a maf file until a certain number of columns
have been passed.

usage: %prog -c cols < maf > maf
"""

import sys
from optparse import OptionParser

from bx.align import maf


def __main__():
    parser = OptionParser()
    parser.add_option("-c", "--cols", action="store")

    (options, args) = parser.parse_args()

    maf_reader = maf.Reader(sys.stdin)
    maf_writer = maf.Writer(sys.stdout)

    if not options.cols:
        raise Exception("Cols argument is required")
    cols = int(options.cols)

    count = 0

    for m in maf_reader:

        maf_writer.write(m)

        count += m.text_size

        if count >= cols:
            return


if __name__ == "__main__":
    __main__()



# cython: profile=False

"""
Core implementation for reading UCSC "big binary indexed" files.

There isn't really any specification for the format beyond the code, so this
mirrors Jim Kent's 'bbiRead.c' mostly. 
"""

from cpython.version cimport PY_MAJOR_VERSION
import sys

cimport cython

from collections import deque
from bpt_file cimport BPTFile
from cirtree_file cimport CIRTreeFile
from types cimport *

from libc cimport limits

import numpy
cimport numpy

from bx.misc.binary_file import BinaryFileReader
from io import BytesIO
import zlib, math

cdef extern from "Python.h":
    char * PyBytes_AsString( object )

# Signatures for bbi related file types

cdef public int big_wig_sig = 0x888FFC26
cdef public int big_bed_sig = 0x8789F2EB

# Some record sizes for parsing
DEF summary_on_disk_size = 32

@cython.profile(False)
cdef inline int range_intersection( int start1, int end1, int start2, int end2 ):
    return min( end1, end2 ) - max( start1, start2 )

@cython.profile(False)
cdef inline int imax(int a, int b): return a if a >= b else b
@cython.profile(False)
cdef inline int imin(int a, int b): return a if a <= b else b

cdef enum summary_type:
    summary_type_mean = 0
    summary_type_max = 1
    summary_type_min = 2
    summary_type_coverage = 3
    summary_type_sd = 4

cdef class SummaryBlock:
    """
    A block of summary data from disk
    """
    pass

cdef class SummarizedData:
    """
    The result of using SummaryBlocks read from the file to produce a 
    aggregation over a particular range and resolution
    """
    def __init__( self, bits32 start, bits32 end, int size ):
        self.start = start
        self.end = end
        self.size = size
        self.valid_count = numpy.zeros( self.size, dtype=numpy.float64 )
        self.min_val = numpy.zeros( self.size, dtype=numpy.float64 )
        self.max_val = numpy.zeros( self.size, dtype=numpy.float64 )
        self.sum_data = numpy.zeros( self.size, dtype=numpy.float64 )
        self.sum_squares = numpy.zeros( self.size, dtype=numpy.float64 )
    cdef accumulate_interval_value( self, bits32 s, bits32 e, float val ):
        cdef int base_start, base_end, base_step, overlap, j, interval_size
        cdef double overlap_factor, interval_weight
        # We locally cdef the arrays so all indexing will be at C speeds
        cdef numpy.ndarray[numpy.float64_t, ndim=1] valid_count = self.valid_count
        cdef numpy.ndarray[numpy.float64_t, ndim=1] min_val = self.min_val
        cdef numpy.ndarray[numpy.float64_t, ndim=1] max_val = self.max_val
        cdef numpy.ndarray[numpy.float64_t, ndim=1] sum_data = self.sum_data
        cdef numpy.ndarray[numpy.float64_t, ndim=1] sum_squares = self.sum_squares
        # Trim interval down to region of interest
        if s < self.start: 
            s = self.start
        if e > self.end: 
            e = self.end
        if s >= e: 
            return
        base_step = ( self.end - self.start ) / self.size
        for j from 0 <= j < self.size:
            base_start = self.start + ( base_step * j )
            base_end = base_start + base_step
            overlap = range_intersection( base_start, base_end, s, e )
            if overlap > 0:
                interval_size = e - s
                overlap_factor = <double> overlap / interval_size
                interval_weight = interval_size * overlap_factor
                valid_count[j] += interval_weight
                sum_data[j] += val * interval_weight
                sum_squares[j] += val * val * interval_weight
                if max_val[j] < val:
                    max_val[j] = val
                if min_val[j] > val:
                    min_val[j] = val 

cdef class BlockHandler:
    """
    Callback for `BBIFile.visit_blocks_in_region`
    """
    cdef handle_block( self, bytes block_data, BBIFile bbi_file ):
        pass

cdef class BBIFile:
    """
    A "big binary indexed" file. Stores blocks of raw data and numeric 
    summaries of that data at different levels of aggregation ("zoom levels").
    Generic enough to accommodate both wiggle and bed data. 
    """

    def __init__( self, file=None, expected_sig=None, type_name=None ):
        if file is not None:
            self.open( file, expected_sig, type_name )

    def open( self, file, expected_sig, type_name ):
        """
        Initialize from an existing bbi file, signature (magic) must be passed
        in since this is generic. 
        """
        assert expected_sig is not None
        self.file = file
        # Open the file in a BinaryFileReader, handles magic and byteswapping
        self.reader = reader = BinaryFileReader( file, expected_sig )
        self.magic = expected_sig
        self.is_byteswapped = self.reader.byteswap_needed
        # Read header stuff
        self.version = reader.read_uint16()
        self.zoom_levels = reader.read_uint16()
        self.chrom_tree_offset = reader.read_uint64()
        self.unzoomed_data_offset = reader.read_uint64()
        self.unzoomed_index_offset = reader.read_uint64()
        self.field_count = reader.read_uint16()
        self.defined_field_count = reader.read_uint16()
        self.as_offset = reader.read_uint64()
        self.total_summary_offset = reader.read_uint64()
        self.uncompress_buf_size = reader.read_uint32()
        # Skip reserved
        reader.seek( 64 )
        # Read zoom headers
        self.level_list = []
        for i from 0 <= i < self.zoom_levels:
            level = ZoomLevel() 
            level.bbi_file = self
            level.reduction_level = reader.read_uint32()
            level.reserved = reader.read_uint32()
            level.data_offset = reader.read_uint64()
            level.index_offset = reader.read_uint64()
            self.level_list.append( level )
        # Initialize and attach embedded BPTFile containing chromosome names and ids
        reader.seek( self.chrom_tree_offset )
        self.chrom_bpt = BPTFile( file=self.file )

    cdef visit_blocks_in_region( self, bits32 chrom_id, bits32 start, bits32 end, BlockHandler handler ):
        """
        Visit each block from the full data that overlaps a specific region
        """
        cdef CIRTreeFile ctf
        reader = self.reader
        reader.seek( self.unzoomed_index_offset )
        ctf = CIRTreeFile( reader.file )
        block_list = ctf.find_overlapping_blocks( chrom_id, start, end )
        for offset, size in block_list:
            # Seek to and read all data for the block
            reader.seek( offset )
            block_data = reader.read( size )
            # Might need to uncompress
            if self.uncompress_buf_size > 0:
                block_data = zlib.decompress( block_data )
            handler.handle_block( block_data, self )
        
    cpdef summarize( self, object chrom, bits32 start, bits32 end, int summary_size ):
        """
        Gets `summary_size` data points over the regions `chrom`:`start`-`end`.
        """
        cdef char * cchrom
        if PY_MAJOR_VERSION >= 3:
            bytes_chrom = chrom.encode()
        else:
            bytes_chrom = chrom
        cchrom = PyBytes_AsString(bytes_chrom)
        if start >= end:
            return None
        chrom_id, chrom_size = self._get_chrom_id_and_size( cchrom )
        if chrom_id is None:
            return None
        # Return value will be a structured array (rather than an array
        # of summary element structures

        # Find appropriate zoom level
        cdef bits32 base_size = end - start
        cdef int full_reduction = base_size / summary_size
        cdef int zoom = full_reduction / 2
        if zoom < 0:
            zoom = 0
        cdef ZoomLevel zoom_level = self._best_zoom_level( zoom )
        if zoom_level is not None:
            return zoom_level._summarize( chrom_id, start, end, summary_size )
        else:
            return self._summarize_from_full( chrom_id, start, end, summary_size )

    cpdef summarize_from_full( self, char * chrom, bits32 start, bits32 end, int summary_size ):
        """
        Gets `summary_size` data points over the regions `chrom`:`start`-`end`, 
        always using the raw data points
        """
        if start >= end:
            return None
        chrom_id, chrom_size = self._get_chrom_id_and_size( chrom )
        if chrom_id is None:
            return None
        # Return value will be a structured array (rather than an array
        # of summary element structures
        return self._summarize_from_full( chrom_id, start, end, summary_size )

    cpdef query( self, object chrom, bits32 start, bits32 end, int summary_size ):
        """
        Provides a different view of summary for region, a list of dictionaries
        with keys: mean, max, min, coverage, std_dev
        """

        if end > 2147483647 or start < 0:
            raise ValueError
        results = self.summarize(chrom, start, end, summary_size)

        if not results:
            return None
        
        rval = []
        for i in range(summary_size):
            sum_data = results.sum_data[i]
            valid_count = results.valid_count[i]
            mean = sum_data / valid_count
            coverage = <double> summary_size / (end - start) * valid_count
            
            # print results.sum_squares[i], sum_data, valid_count
            variance = results.sum_squares[i] - sum_data * sum_data / valid_count
            if valid_count > 1:
                variance /= valid_count - 1
            std_dev = math.sqrt(max(variance, 0))

            rval.append( { "mean": mean, "max": results.max_val[i], "min": results.min_val[i], \
                        "coverage": coverage, "std_dev": std_dev } )
        
        return rval

    cdef _get_chrom_id_and_size( self, char * chrom ):
        """
        Lookup id and size from the chromosome named `chrom`
        """
        bytes = self.chrom_bpt.find( chrom )
        if bytes is not None:
            # The value is two 32 bit uints, use the BPT's reader for checking byteswapping
            assert len( bytes ) == 8
            chrom_id, chrom_size = self.chrom_bpt.reader.unpack( "II", bytes )
            return chrom_id, chrom_size
        else:
            return None, None
    
    cdef _summarize_from_full( self, bits32 chrom_id, bits32 start, bits32 end, int summary_size ):
        """
        Create summary from full data. This is data specific so must be overridden.
        """
        pass
        
    cdef _best_zoom_level( self, int desired_reduction ):
        if desired_reduction <= 1:
            return None
            
        cdef ZoomLevel level, closest_level
        cdef int diff, closest_diff = limits.INT_MAX
        
        closest_level = None
        for level in self.level_list:
            diff = desired_reduction - level.reduction_level
            if diff >= 0 and diff < closest_diff:
                closest_diff = diff
                closest_level = level
        return closest_level

cdef class ZoomLevel:
    cdef BBIFile bbi_file
    cdef public bits32 reduction_level
    cdef bits32 reserved
    cdef public bits64 data_offset
    cdef public bits64 index_offset
    cdef int item_count

    def _summary_blocks_in_region( self, bits32 chrom_id, bits32 start, bits32 end ):
        """
        Return a list of all SummaryBlocks that overlap the region 
        `chrom_id`:`start`-`end`
        """
        cdef CIRTreeFile ctf
        cdef SummaryBlock summary
        rval = deque()
        reader = self.bbi_file.reader
        reader.seek( self.index_offset )
        ctf = CIRTreeFile( reader.file )
        block_list = ctf.find_overlapping_blocks( chrom_id, start, end )
        for offset, size in block_list:
            # Seek to and read all data for the block
            reader.seek( offset )
            block_data = reader.read( size )
            # Might need to uncompress
            if self.bbi_file.uncompress_buf_size > 0:
                ## block_data = zlib.decompress( block_data, buf_size = self.bbi_file.uncompress_buf_size )
                block_data = zlib.decompress( block_data )
            block_size = len( block_data )
            # The block should be a bunch of summaries. 
            assert block_size % summary_on_disk_size == 0
            item_count = block_size / summary_on_disk_size
            # Create another reader just for the block, shouldn't be too expensive
            block_reader = BinaryFileReader( BytesIO( block_data ), is_little_endian=reader.is_little_endian )
            for i from 0 <= i < item_count:
                ## NOTE: Look carefully at bbiRead again to be sure the endian
                ##       conversion here is all correct. It looks like it is 
                ##       just pushing raw data into memory and not swapping
                
                sum_chrom_id = block_reader.read_uint32()
                # A block can contain summaries from more that one chrom_id
                if sum_chrom_id != chrom_id:
                    block_reader.skip(7*4)
                    continue
                       
                summary = SummaryBlock()
                summary.chrom_id = sum_chrom_id
                summary.start = block_reader.read_uint32()
                summary.end = block_reader.read_uint32()
                summary.valid_count = block_reader.read_uint32()
                summary.min_val = block_reader.read_float()
                summary.max_val = block_reader.read_float()
                summary.sum_data = block_reader.read_float()
                summary.sum_squares = block_reader.read_float()
                rval.append( summary )
                

        return rval
    
    cdef _get_summary_slice( self, bits32 base_start, bits32 base_end, summaries ):
        cdef float valid_count = 0.0
        cdef float sum_data = 0.0
        cdef float sum_squares = 0.0
        cdef float min_val = numpy.nan
        cdef float max_val = numpy.nan
        cdef float overlap_factor
        cdef int overlap
        
        if summaries:
            
            min_val = summaries[0].min_val
            max_val = summaries[0].max_val
            
            for summary in summaries:
                if summary.start >= base_end:
                    break
            
                overlap = range_intersection( base_start, base_end, summary.start, summary.end )
                if overlap > 0:
                    overlap_factor = <double> overlap / (summary.end - summary.start)
                
                    valid_count += summary.valid_count * overlap_factor
                    sum_data += summary.sum_data * overlap_factor
                    sum_squares += summary.sum_squares * overlap_factor

                    if max_val < summary.max_val:
                        max_val = summary.max_val
                    if min_val > summary.min_val:
                        min_val = summary.min_val

        return valid_count, sum_data, sum_squares, min_val, max_val
        
    cdef _summarize( self, bits32 chrom_id, bits32 start, bits32 end, int summary_size ):
        """
        Summarize directly from file. 

        Looking at Jim's code, it appears that 
          - bbiSummariesInRegion returns all summaries that span start-end in 
            sorted order
          - bbiSummarySlice is then used to aggregate over the subset of those 
            summaries that overlap a single summary element
        """
        cdef bits32 base_start, base_end, base_step
        
        # We locally cdef the arrays so all indexing will be at C speeds
        cdef numpy.ndarray[numpy.float64_t] valid_count
        cdef numpy.ndarray[numpy.float64_t] min_val
        cdef numpy.ndarray[numpy.float64_t] max_val
        cdef numpy.ndarray[numpy.float64_t] sum_data
        cdef numpy.ndarray[numpy.float64_t] sum_squares
        
        # What we will load into
        rval = SummarizedData( start, end, summary_size )
        valid_count = rval.valid_count
        min_val = rval.min_val
        max_val = rval.max_val
        sum_data = rval.sum_data
        sum_squares = rval.sum_squares
        # First, load up summaries
        reader = self.bbi_file.reader
        reader.seek( self.index_offset )
        summaries = self._summary_blocks_in_region(chrom_id, start, end)

        base_step = (end - start) / summary_size
        base_start = start
        base_end = start
        
        for i in range(summary_size):
            base_end += base_step
            
            while summaries and summaries[0].end <= base_start:
                summaries.popleft()

            valid_count[i], sum_data[i], sum_squares[i], min_val[i], max_val[i] = self._get_summary_slice(base_start, base_end, summaries)
            base_start = base_end
        
        return rval



"""
Tools for tiling / projecting alignments onto an interval of a sequence.
"""

import bx.seq.nib


def tile_interval(sources, index, ref_src, start, end, seq_db=None):
    """
    Tile maf blocks onto an interval. The resulting block will span the interval
    exactly and contain the column from the highest scoring alignment at each
    position.

    `sources`: list of sequence source names to include in final block
    `index`: an instnace that can return maf blocks overlapping intervals
    `ref_src`: source name of the interval (ie, hg17.chr7)
    `start`: start of interval
    `end`: end of interval
    `seq_db`: a mapping for source names in the reference species to nib files
    """
    # First entry in sources should also be on the reference species
    assert sources[0].split('.')[0] == ref_src.split('.')[0], \
        "%s != %s" % (sources[0].split('.')[0], ref_src.split('.')[0])
    base_len = end - start
    blocks = index.get(ref_src, start, end)
    # From low to high score
    blocks.sort(key=lambda t: t.score)
    mask = [-1] * base_len
    for i, block in enumerate(blocks):
        ref = block.get_component_by_src_start(ref_src)
        assert ref.strand == "+"
        slice_start = max(start, ref.start)
        slice_end = min(end, ref.end)
        for j in range(slice_start, slice_end):
            mask[j-start] = i
    tiled = []
    for i in range(len(sources)):
        tiled.append([])
    for ss, ee, index in intervals_from_mask(mask):
        # Interval with no covering alignments
        if index < 0:
            # Get sequence if available, otherwise just use 'N'
            if seq_db:
                tiled[0].append(bx.seq.nib.NibFile(open(seq_db[ref_src])).get(start+ss, ee-ss))
            else:
                tiled[0].append("N" * (ee-ss))
            # Gaps in all other species
            for row in tiled[1:]:
                row.append("-" * (ee - ss))
        else:
            slice_start = start + ss
            slice_end = start + ee
            block = blocks[index]
            ref = block.get_component_by_src_start(ref_src)
            sliced = block.slice_by_component(ref, slice_start, slice_end)
            sliced = sliced.limit_to_species(sources)
            sliced.remove_all_gap_columns()
            for i, src in enumerate(sources):
                comp = sliced.get_component_by_src_start(src)
                if comp:
                    tiled[i].append(comp.text)
                else:
                    tiled[i].append("-" * sliced.text_size)
    return ["".join(t) for t in tiled]


def intervals_from_mask(mask):
    start = 0
    last = mask[0]
    for i in range(1, len(mask)):
        if mask[i] != last:
            yield start, i, last
            start = i
            last = mask[i]
    yield start, len(mask), last



#!/usr/bin/env python2.4
"""
For each interval in `bed1` count the number of intersecting regions in `bed2`.

usage: %prog bed1 bed2
"""
from __future__ import print_function

import sys

from bx.intervals import (
    Intersecter,
    Interval
)

bed1, bed2 = sys.argv[1:3]

ranges = {}
for line in open(bed2):
    fields = line.strip().split()
    chrom, start, end, = fields[0], int(fields[1]), int(fields[2])
    if chrom not in ranges:
        ranges[chrom] = Intersecter()
    ranges[chrom].add_interval(Interval(start, end))

for line in open(bed1):
    fields = line.strip().split()
    chrom, start, end = fields[0], int(fields[1]), int(fields[2])
    other = " ".join(fields[3:])
    out = " ".join(fields[:3] + [other])
    if chrom in ranges:
        print(out, len(ranges[chrom].find(start, end)))
    else:
        print(out, 0)



#!/usr/bin/env python

"""
Read a file from stdin, split each line and write fields one per line to
stdout.

TODO: is this really that useful?
"""
from __future__ import print_function

import sys

for line in sys.stdin:
    for field in line.split():
        print(field)



#!/usr/bin/env python2.4
"""
Create a bed file listing all the divergent sites between two specific species
in a maf.

usage: %prog maf_file reference_species_name other_species_name
"""
from __future__ import print_function

import sys

import bx.align.maf
import bx.bitset


def main():
    bitsets = {}
    maf = sys.argv[1]
    reference_sp, other_sp = sys.argv[2], sys.argv[3]

    for block in bx.align.maf.Reader(open(maf)):
        ref = block.get_component_by_src_start(reference_sp)
        other = block.get_component_by_src_start(other_sp)

        if not ref or not other:
            continue
        ref_chrom = ref.src.split('.')[1]
        ref_start = ref.start
        chrom_size = ref.get_src_size()

        if ref_chrom not in bitsets:
            bitsets[ref_chrom] = bx.bitset.BinnedBitSet(chrom_size)

        pos = ref_start
        for i, j in zip(ref.text.upper(), other.text.upper()):
            if i != '-':
                if i != j:  # mismatch
                    if i != 'N' and j != 'N' and j != '-':
                        # set if all valid chars
                        bitsets[ref_chrom].set(pos)
                pos += 1

    # bits --> bed file
    for chrom in bitsets:
        bits = bitsets[chrom]
        end = 0
        while True:
            start = bits.next_set(end)
            if start == bits.size:
                break
            end = bits.next_clear(start)
            print("%s\t%d\t%d" % (chrom, start, end))


main()



"""
Support for reading and writing genomic intervals from delimited text files.
"""
from __future__ import print_function

from bx.bitset import (
    BinnedBitSet,
    MAX
)
from bx.tabular.io import (
    ParseError,
    TableReader,
    TableRow,
)


class MissingFieldError(ParseError):
    pass


class FieldFormatError(ParseError):
    def __init__(self, *args, **kwargs):
        ParseError.__init__(self, *args, **kwargs)
        self.expected = kwargs.get("expected", None)

    def __str__(self):
        if self.expected:
            return ParseError.__str__(self) + ", " + self.expected + " expected"
        else:
            return ParseError.__str__(self)


class StrandFormatError(ParseError):
    pass


class GenomicInterval(TableRow):
    """
    A genomic interval stored in a set of fields (a row of a table)
    """

    def __init__(self, reader, fields, chrom_col, start_col, end_col, strand_col, default_strand, fix_strand=False):
        TableRow.__init__(self, reader, fields)
        self.chrom_col = chrom_col
        self.start_col = start_col
        self.end_col = end_col
        self.strand_col = strand_col
        self.nfields = nfields = len(fields)
        # Parse chrom/source column
        if chrom_col >= nfields:
            raise MissingFieldError("No field for chrom_col (%d)" % chrom_col)
        self.chrom = fields[chrom_col].strip()
        # Parse start column and ensure it is an integer
        if start_col >= nfields:
            raise MissingFieldError("No field for start_col (%d)" % start_col)
        try:
            self.start = int(fields[start_col])
        except ValueError as e:
            raise FieldFormatError("Could not parse start_col: " + str(e), expected="integer")
        # Parse end column and ensure it is an integer
        if end_col >= nfields:
            raise MissingFieldError("No field for end_col (%d)" % end_col)
        try:
            self.end = int(fields[end_col])
        except ValueError as e:
            raise FieldFormatError("Could not parse end_col: " + str(e), expected="integer")
        # Ensure start <= end
        if self.end < self.start:
            raise ParseError("Start is greater than End. Interval length is < 1.")
        # Parse strand and ensure it is valid
        if strand_col >= nfields or strand_col < 0:
            # This should probable be immutable since the fields are
            # not updated when it is set
            self.strand = default_strand
        else:
            strand = fields[strand_col]
            if strand == ".":
                strand = default_strand
            elif strand not in ("+", "-"):
                if fix_strand:
                    strand = "+"
                else:
                    raise StrandFormatError("Strand must be either '+' or '-'")
            self.strand = strand

    def __setattr__(self, name, value):
        if name == "chrom":
            self.fields[self.chrom_col] = str(value)
        elif name == "start":
            self.fields[self.start_col] = str(value)
        elif name == "end":
            self.fields[self.end_col] = str(value)
        elif name == "strand":
            if self.strand_col < self.nfields and self.strand_col >= 0:
                self.fields[self.strand_col] = str(value)
        object.__setattr__(self, name, value)

    def __str__(self):
        return "\t".join(self.fields)

    def copy(self):
        return GenomicInterval(self.reader, list(self.fields), self.chrom_col, self.start_col, self.end_col, self.strand_col, self.strand)


class GenomicIntervalReader(TableReader):
    """
    Reader for iterating a set of intervals in a tab separated file. Can
    also parse header and comment lines if requested.

    >>> from bx.tabular.io import Comment, Header
    >>> r = GenomicIntervalReader( [ "#chrom\\tname\\tstart\\tend\\textra",
    ...               "chr1\\tfoo\\t1\\t100\\txxx",
    ...               "chr2\\tbar\\t20\\t300\\txxx",
    ...               "#I am a comment",
    ...               "chr2\\tbar\\t20\\t300\\txxx" ], start_col=2, end_col=3 )
    >>> header = next(r)
    >>> elements = list(r)
    >>> elements.insert(0, header)
    >>> assert isinstance(elements[0], Header)
    >>> str(elements[0])
    '#chrom\\tname\\tstart\\tend\\textra'
    >>> assert isinstance(elements[1], GenomicInterval)
    >>> print(elements[1].start, elements[1].end)
    1 100
    >>> str(elements[1])
    'chr1\\tfoo\\t1\\t100\\txxx'
    >>> elements[1].start = 30
    >>> print(elements[1].start, elements[1].end)
    30 100
    >>> str(elements[1])
    'chr1\\tfoo\\t30\\t100\\txxx'
    >>> assert isinstance(elements[2], GenomicInterval)
    >>> assert isinstance(elements[3], Comment)
    >>> assert isinstance(elements[4], GenomicInterval)
    """

    def __init__(self, input, chrom_col=0, start_col=1, end_col=2, strand_col=5,
                 default_strand="+", return_header=True, return_comments=True, force_header=None, fix_strand=False, comment_lines_startswith=["#", "track "], allow_spaces=False):
        TableReader.__init__(self, input, return_header, return_comments, force_header, comment_lines_startswith)
        self.chrom_col = chrom_col
        self.start_col = start_col
        self.end_col = end_col
        self.strand_col = strand_col
        self.default_strand = default_strand
        self.fix_strand = fix_strand
        self.allow_spaces = allow_spaces

    def parse_row(self, line):
        # Try multiple separators. First tab, our expected splitter, than
        # just whitespace in the case of problematic files with space instead of
        # tab separation
        seps = ["\t"]
        if self.allow_spaces:
            seps.append(None)
        for i, sep in enumerate(seps):
            try:
                return GenomicInterval(
                    self, line.split(sep), self.chrom_col, self.start_col,
                    self.end_col, self.strand_col, self.default_strand,
                    fix_strand=self.fix_strand)
            except Exception as e:
                # Catch and store the initial error
                if i == 0:
                    err = e
        # Ran out of separators and still have errors, raise our problem
        raise err

    def binned_bitsets(self, upstream_pad=0, downstream_pad=0, lens={}):
        # The incoming lens dictionary is a dictionary of chromosome lengths
        # which are used to initialize the bitsets.
        last_chrom = None
        last_bitset = None
        bitsets = dict()
        for interval in self:
            if isinstance(interval, GenomicInterval):
                chrom = interval[self.chrom_col]
                if chrom != last_chrom:
                    if chrom not in bitsets:
                        size = lens.get(chrom, MAX)
                        try:
                            bbs = BinnedBitSet(size)
                        except ValueError as e:
                            # We will only reach here when constructing this bitset from the lens dict
                            # since the value of MAX is always safe.
                            raise Exception("Invalid chrom length %s in 'lens' dictionary. %s" % (str(size), str(e)))
                        bitsets[chrom] = bbs
                    last_chrom = chrom
                    last_bitset = bitsets[chrom]
                start = max(int(interval[self.start_col]), 0)
                end = min(int(interval[self.end_col]), last_bitset.size)
                last_bitset.set_range(start, end-start)
        return bitsets


class NiceReaderWrapper(GenomicIntervalReader):
    """
    >>> from bx.tabular.io import Header
    >>> r = NiceReaderWrapper(["#chrom\\tname\\tstart\\tend\\textra",
    ...                        "chr1\\tfoo\\t1\\t100\\txxx",
    ...                        "chr2\\tbar\\t20\\t300\\txxx",
    ...                        "#I am a comment",
    ...                        "chr2\\tbar\\t20\\t300\\txxx" ], start_col=2, end_col=3 )
    >>> assert isinstance(next(r), Header)
    >>> assert r.current_line == '#chrom\\tname\\tstart\\tend\\textra', r.current_line
    >>> assert len([_ for _ in r]) == 4
    """

    def __init__(self, reader, **kwargs):
        GenomicIntervalReader.__init__(self, reader, **kwargs)
        self.outstream = kwargs.get("outstream", None)
        self.print_delegate = kwargs.get("print_delegate", None)
        self.input_wrapper = iter(self.input)
        self.input_iter = self.iterwrapper()
        self.skipped = 0
        self.skipped_lines = []

    def __iter__(self):
        return self

    def __next__(self):
        while True:
            try:
                nextitem = super(NiceReaderWrapper, self).__next__()
                return nextitem
            except ParseError as e:
                if self.outstream:
                    if self.print_delegate and hasattr(self.print_delegate, "__call__"):
                        self.print_delegate(self.outstream, e, self)
                self.skipped += 1
                # no reason to stuff an entire bad file into memory
                if self.skipped < 10:
                    self.skipped_lines.append((self.linenum, self.current_line, str(e)))

    def iterwrapper(self):
        # Generator which keeps track of the current line as an object attribute.
        for self.current_line in self.input_wrapper:
            yield self.current_line


class BitsetSafeReaderWrapper(NiceReaderWrapper):
    def __init__(self, reader, lens={}):
        # This class handles any ValueError, IndexError and OverflowError exceptions that may be thrown when
        # the bitsets are being created by skipping the problem lines.
        # The incoming lens dictionary is a dictionary of chromosome lengths
        # which are used to initialize the bitsets.
        # It is assumed that the reader is an interval reader, i.e. it has chr_col, start_col, end_col and strand_col attributes.
        NiceReaderWrapper.__init__(self, reader.input, chrom_col=reader.chrom_col, start_col=reader.start_col, end_col=reader.end_col, strand_col=reader.strand_col)
        self.lens = lens

    def __next__(self):
        while True:
            rval = super(BitsetSafeReaderWrapper, self).__next__()
            if isinstance(rval, GenomicInterval) and rval.end > self.lens.get(rval.chrom, MAX):
                self.skipped += 1
                # no reason to stuff an entire bad file into memory
                if self.skipped < 10:
                    self.skipped_lines.append((self.linenum, self.current_line, "Error in BitsetSafeReaderWrapper"))
            else:
                return rval



"""
Access to files containing sequence data in 'twobit' format.
"""
from struct import calcsize, unpack

from six.moves.collections_abc import Mapping

from . import _twobit

TWOBIT_MAGIC_NUMBER = 0x1A412743
TWOBIT_MAGIC_NUMBER_SWAP = 0x4327411A
TWOBIT_MAGIC_SIZE = 4

TWOBIT_VERSION = 0


class TwoBitSequence(object):
    def __init__(self, tbf, header_offset=None):
        self.tbf = tbf
        self.header_offset = header_offset
        self.sequence_offset = None
        self.size = None
        self.n_blocks = None
        self.masked_blocks = None
        self.loaded = False

    def __getitem__(self, slice):
        start, stop, stride = slice.indices(self.size)
        assert stride == 1, "Striding in slices not supported"
        if stop - start < 1:
            return ""
        return _twobit.read(self.tbf.file, self, start, stop, self.tbf.do_mask)

    def __len__(self):
        return self.size

    def get(self, start, end):
        # Trim start / stop
        if start < 0:
            start = 0
        if end > self.size:
            end = self.size
        out_size = end - start
        if out_size < 1:
            raise Exception("end before start (%s,%s)" % (start, end))
        # Find position of packed portion
        dna = _twobit.read(self.tbf.file, self, start, end, self.tbf.do_mask)
        # Return
        return dna


class TwoBitFile(Mapping):
    def __init__(self, file, do_mask=True):
        self.do_mask = do_mask
        # Read magic and determine byte order
        self.byte_order = ">"
        strng = file.read(TWOBIT_MAGIC_SIZE)
        magic = unpack(">L", strng)[0]
        if magic != TWOBIT_MAGIC_NUMBER:
            if magic == TWOBIT_MAGIC_NUMBER_SWAP:
                self.byte_order = "<"
            else:
                raise Exception("Not a NIB file")
        self.magic = magic
        self.file = file
        # Read version
        self.version = self.read("L")
        if self.version != TWOBIT_VERSION:
            raise Exception("File is version '%d' but I only know about '%d'" % (self.version, TWOBIT_VERSION))
        # Number of sequences in file
        self.seq_count = self.read("L")
        # Header contains some reserved space
        self.reserved = self.read("L")
        # Read index of sequence names to offsets
        index = dict()
        for _ in range(self.seq_count):
            name = self.read_p_string()
            offset = self.read("L")
            index[name] = TwoBitSequence(self, offset)
        self.index = index

    def __getitem__(self, name):
        seq = self.index[name]
        if not seq.loaded:
            self.load_sequence(name)
        return seq

    def __iter__(self):
        return iter(self.index.keys())

    def __len__(self):
        return len(self.index)

    def load_sequence(self, name):
        seq = self.index[name]
        # Seek to start of sequence block
        self.file.seek(seq.header_offset)
        # Size of sequence
        seq.size = self.read("L")
        # Read N and masked block regions
        seq.n_block_starts, seq.n_block_sizes = self.read_block_coords()
        seq.masked_block_starts, seq.masked_block_sizes = self.read_block_coords()
        # Reserved
        self.read("L")
        # Save start of actualt sequence
        seq.sequence_offset = self.file.tell()
        # Mark as loaded
        seq.loaded = True

    def read_block_coords(self):
        block_count = self.read("L")
        if block_count == 0:
            return [], []
        starts = self.read(str(block_count) + "L", untuple=False)
        sizes = self.read(str(block_count) + "L", untuple=False)
        return list(starts), list(sizes)

    def read(self, pattern, untuple=True):
        rval = unpack(self.byte_order + pattern,
                      self.file.read(calcsize(self.byte_order + pattern)))
        if untuple and len(rval) == 1:
            return rval[0]
        return rval

    def read_p_string(self):
        """
        Read a length-prefixed string
        """
        length = self.read("B")
        return self.file.read(length)



#!/usr/bin/env python

"""
Filter each block in a maf file. Can filter blocks for a minimum number of
components (rows), a minimum length in columns, or an arbitrary python
expression (which will be evaluated for each block with the variable 'm'
containing that block).

usage: %prog [options] < maf > maf
    --component_count=N: Minimum number of components (rows)
    --min_cols=N: Minimum number of columns
    -e, --expr=EXPR: Python expression that must evaulate to true
"""

import sys
from optparse import OptionParser

from bx.align import maf


def __main__():

    # Parse command line arguments

    parser = OptionParser()
    parser.add_option("--component_count", action="store", default=None, type="int", help="")
    parser.add_option("--min_cols", action="store", default=None, type="int", help="")
    parser.add_option("-e", "--expr", action="store", default=None)

    (options, args) = parser.parse_args()

    component_count = options.component_count
    min_cols = options.min_cols
    expr = options.expr

    # Compile expression for SPEED
    if expr:
        expr = compile(expr, '<expr arg>', 'eval')

    maf_reader = maf.Reader(sys.stdin)
    maf_writer = maf.Writer(sys.stdout)

    for m in maf_reader:

        if component_count and len(m.components) != component_count:
            continue
        if min_cols and m.text_size < min_cols:
            continue
        if expr and not bool(eval(expr, {"m": m, "maf": m})):
            continue

        maf_writer.write(m)


if __name__ == "__main__":
    __main__()



"""
BigWig file.
"""

from collections import deque
from bbi_file cimport *
from cirtree_file cimport CIRTreeFile
import numpy
cimport numpy
from types cimport *
from bx.misc.binary_file import BinaryFileReader
from io import BytesIO
import zlib

DEF big_wig_sig = 0x888FFC26
DEF bwg_bed_graph = 1
DEF bwg_variable_step = 2
DEF bwg_fixed_step = 3

cdef inline int range_intersection( int start1, int end1, int start2, int end2 ):
    return min( end1, end2 ) - max( start1, start2 )

cdef class BigWigBlockHandler( BlockHandler ):
    """
    BlockHandler that parses the block into a series of wiggle records, and calls `handle_interval_value` for each.
    """
    cdef bits32 start
    cdef bits32 end
    def __init__( self, bits32 start, bits32 end ):
        BlockHandler.__init__( self )
        self.start = start
        self.end = end
    cdef handle_block( self, bytes block_data, BBIFile bbi_file ):
        cdef bits32 b_chrom_id, b_start, b_end, b_valid_count
        cdef bits32 b_item_step, b_item_span
        cdef bits16 b_item_count
        cdef UBYTE b_type
        cdef int s, e
        cdef float val
        # Now we parse the block, first the header
        block_reader = BinaryFileReader( BytesIO( block_data ), is_little_endian=bbi_file.reader.is_little_endian )
        b_chrom_id = block_reader.read_uint32()
        b_start = block_reader.read_uint32()
        b_end = block_reader.read_uint32()
        b_item_step = block_reader.read_uint32()
        b_item_span = block_reader.read_uint32()
        b_type = block_reader.read_uint8()
        block_reader.skip(1)
        b_item_count = block_reader.read_uint16()
        for i from 0 <= i < b_item_count:
            # Depending on the type, s and e are either read or 
            # generate using header, val is always read
            if b_type == bwg_bed_graph: 
                s = block_reader.read_uint32()
                e = block_reader.read_uint32()
                val = block_reader.read_float()
            elif b_type == bwg_variable_step:
                s = block_reader.read_uint32()
                e = s + b_item_span
                val = block_reader.read_float()
            elif b_type == bwg_fixed_step:
                s = b_start + ( i * b_item_span )
                e = s + b_item_span
                val = block_reader.read_float()
            else:
                # FIXME: raise exception???
                #        s, e, val are uninitialized/not updated at this point!
                pass
            if s < self.start: 
                s = self.start
            if e > self.end: 
                e = self.end
            if s >= e: 
                continue
            self.handle_interval_value( s, e, val )

    cdef handle_interval_value( self, bits32 s, bits32 e, float val ):
        pass

cdef class SummarizingBlockHandler( BigWigBlockHandler ):
    """
    Accumulates intervals into a SummarizedData
    """
    cdef SummarizedData sd
    def __init__( self, bits32 start, bits32 end, int summary_size ):
        BigWigBlockHandler.__init__( self, start, end )
        # What we will load into
        self.sd = SummarizedData( start, end, summary_size )
        for i in range(summary_size):
            self.sd.min_val[i] = +numpy.inf
        for i in range(summary_size):
            self.sd.max_val[i] = -numpy.inf

    cdef handle_interval_value( self, bits32 s, bits32 e, float val ):
        self.sd.accumulate_interval_value( s, e, val )

cdef class IntervalAccumulatingBlockHandler( BigWigBlockHandler ):
    cdef list intervals
    """
    Accumulates intervals into a list of intervals with values
    """
    def __init__( self, bits32 start, bits32 end ):
        BigWigBlockHandler.__init__( self, start, end )
        self.intervals = []

    cdef handle_interval_value( self, bits32 s, bits32 e, float val ):
        self.intervals.append( ( s, e, val ) )

cdef class ArrayAccumulatingBlockHandler( BigWigBlockHandler ):
    """
    Accumulates intervals into a list of intervals with values
    """
    cdef numpy.ndarray array
    def __init__( self, bits32 start, bits32 end ):
        BigWigBlockHandler.__init__( self, start, end )
        self.array = numpy.zeros( end - start, dtype=numpy.float32 )
        self.array[...] = numpy.nan

    cdef handle_interval_value( self, bits32 s, bits32 e, float val ):
        cdef numpy.ndarray[ numpy.float32_t, ndim=1 ] array = self.array
        cdef int i
        # Slicing is not optimized by Cython
        for i from s - self.start <= i < e - self.start:
            array[ i ] = val

cdef class BigWigHeaderBlockHandler( BigWigBlockHandler ):
    "Reads and returns headers"
    cdef list headers

    def __init__( self, bits32 start, bits32 end ):
        BigWigBlockHandler.__init__( self, start, end )
        self.headers = []

    cdef handle_block( self, bytes block_data, BBIFile bbi_file ):
        cdef bits32 b_chrom_id, b_start, b_end, b_valid_count
        cdef bits32 b_item_step, b_item_span
        cdef bits16 b_item_count
        cdef UBYTE b_type
        cdef int s, e
        cdef float val
        # parse the block header
        block_reader = BinaryFileReader( BytesIO( block_data ), is_little_endian=bbi_file.reader.is_little_endian )
        b_chrom_id = block_reader.read_uint32()
        b_start = block_reader.read_uint32()
        b_end = block_reader.read_uint32()
        b_item_step = block_reader.read_uint32()
        b_item_span = block_reader.read_uint32()
        b_type = block_reader.read_uint8()
        block_reader.skip(1)
        b_item_count = block_reader.read_uint16()
        self.handle_header( b_start, b_end,  b_item_step, b_item_span, b_type, b_item_count )

    cdef handle_header( self, bits32 start, bits32 end, bits32 step, bits32 span, bits8 type, bits16 itemCount ):
        self.headers.append( ( start, end, step, span, type, itemCount ) )

cdef class BigWigFile( BBIFile ):
    """
    A "big binary indexed" file whose raw data is in wiggle format.
    """
    def __init__( self, file=None ):
        BBIFile.__init__( self, file, big_wig_sig, "bigwig" )

    cdef _summarize_from_full( self, bits32 chrom_id, bits32 start, bits32 end, int summary_size ):
        """
        Create summary from full data.
        """
        v = SummarizingBlockHandler( start, end, summary_size )
        self.visit_blocks_in_region( chrom_id, start, end, v )
        # Round valid count, in place
        for i from 0 <= i < summary_size:
            v.sd.valid_count[i] = round( v.sd.valid_count[i] )
        return v.sd

    cpdef get( self, char * chrom, bits32 start, bits32 end ):
        """
        Gets all data points over the regions `chrom`:`start`-`end`.
        """
        if start >= end:
            return None
        chrom_id, chrom_size = self._get_chrom_id_and_size( chrom )
        if chrom_id is None:
            return None
        v = IntervalAccumulatingBlockHandler( start, end )
        self.visit_blocks_in_region( chrom_id, start, end, v )
        return v.intervals

    cpdef get_as_array( self, char * chrom, bits32 start, bits32 end ):
        """
        Gets all data points over the regions `chrom`:`start`-`end`.
        """
        if start >= end:
            return None
        chrom_id, chrom_size = self._get_chrom_id_and_size( chrom )
        if chrom_id is None:
            return None
        v = ArrayAccumulatingBlockHandler( start, end )
        self.visit_blocks_in_region( chrom_id, start, end, v )
        return v.array

    cpdef get_headers( self, char * chrom, bits32 start, bits32 end ):
        if start >= end:
            return None
        chrom_id, chrom_size = self._get_chrom_id_and_size( chrom )
        if chrom_id is None:
            return None
        v = BigWigHeaderBlockHandler( start, end )
        self.visit_blocks_in_region( chrom_id, start, end, v )
        return v.headers









"""
Pyrex extension classes used by `seqmapping.py`.
"""

cdef extern from "stdlib.h":
    void* malloc( size_t )
    void free( void* )

cdef extern from "Python.h":
    int PyObject_AsReadBuffer(object, const void **, Py_ssize_t *) except -1
    int PyObject_AsWriteBuffer(object, void **, Py_ssize_t *) except -1
    int PyBytes_AsStringAndSize(object, char **, Py_ssize_t *) except -1

from numpy import zeros
from math import floor
import random
import sys

cdef class CharToIntArrayMapping:
    """Mapping for converting strings to int arrays"""
    
    cdef int table[256]
    cdef int out_size
    cdef object reverse_table
    
    def __cinit__( self ):
        """Init empty mapping (all characters map to -1)"""
        cdef int i
        for i from 0 <= i < 256: self.table[i] = -1
        self.out_size = 0
        
    def __init__( self ):
        self.reverse_table = dict()

    def set_mapping( self, c, int symbol ):
        """Modify mapping so 'chars' map to 'symbol'"""
        char = ord( c )
        self.table[ char ] = symbol
        if self.out_size <= symbol:
            self.out_size = symbol + 1
        self.reverse_table[ symbol ] = chr( char )

    def translate( self, string ):
        """Translate 'string' and return as int array"""
        cdef Py_ssize_t s_len, t_len
        cdef unsigned char * s_buf
        cdef int * t_buf
        # Get direct access to string
        PyBytes_AsStringAndSize( string, <char **> &s_buf, &s_len )
        # Initialize empty array
        rval = zeros( s_len, 'i' )
        PyObject_AsWriteBuffer( rval, <void **> &t_buf, &t_len ) 
        # Translate
        for i from 0 <= i < s_len:
            t_buf[i] = self.table[ s_buf[ i ] ]
        # Done
        return rval
        
    def translate_list( self, strings ):
        """Translate a list of strings into an int array"""
        cdef Py_ssize_t text_len, i
        cdef Py_ssize_t s_len, t_len
        cdef int factor
        cdef unsigned char * s_buf
        cdef int * t_buf

        # No input, no output
        if len( strings ) < 1: return None
        
        # Length of result
        text_len = len( strings[0] )

        # Init result array
        rval = zeros( text_len, 'i' )
        PyObject_AsWriteBuffer( rval, <void **> &t_buf, &t_len )  

        # Loop over seqs and accumulate result values
        factor = 1
        for string in strings:
            PyBytes_AsStringAndSize( string, <char **> &s_buf, &s_len )
            for i from 0 <= i < text_len:
                if t_buf[i] >= 0: 
                    if self.table[ s_buf[i] ] == -1: 
                        t_buf[i] = -1
                    else: 
                        t_buf[i] = t_buf[i] + ( self.table[ s_buf[i] ] * factor )
            factor = factor * self.out_size
        return rval
        
    def reverse_map( self, val, nseqs ):
        factor = self.out_size ** (nseqs-1)
        rval = []
        while factor > 0:
            rval.append( self.reverse_table[ int( floor( val / factor ) ) ] )
            val = val - ( floor(val/factor) * factor )
            factor = floor( factor / self.out_size )
        rval.reverse()
        return rval
        
    def get_out_size( self ): 
        return self.out_size
         
cdef class IntToIntMapping:
    
    cdef int* table
    cdef int in_size
    cdef int out_size
    
    def __cinit__( self, int in_size ):
        self.in_size = in_size
        self.table = <int *> malloc( in_size * sizeof( int ) )
        if self.table == NULL: raise "Malloc Failed"
        for i from 0 <= i < in_size: self.table[i] = -1
        self.out_size = 0
        
    def __dealloc__( self ):
        # sys.stderr.write( "freeing mapping_helper.IntToIntMapping\n" ); sys.stderr.flush()
        free( self.table )

    def set_mapping( self, int index, int symbol ):
        assert ( -1 <= index < self.in_size ), "%d not between 0 and %s" % ( index, self.in_size )
        self.table[index] = symbol
        if self.out_size <= symbol:
            self.out_size = symbol + 1

    def translate( self, src ):
        """Translate `string` and return as int array"""
        cdef Py_ssize_t s_len, t_len
        cdef int *s_buf
        cdef int *t_buf
        # Get direct access to string
        PyObject_AsReadBuffer( src, <const void **> &s_buf, &s_len )
        s_len = s_len / sizeof( int )
        assert s_len == len( src ), "`src` argument must be a buffer of 32bit integers"
        # Initialize empty array
        rval = zeros( s_len, 'i' )
        PyObject_AsWriteBuffer( rval, <void **> &t_buf, &t_len )
        # Translate
        for i from 0 <= i < s_len:
            if s_buf[i] == -1:
                t_buf[i] = -1
            elif s_buf[i] >= self.in_size:
                t_buf[i] = -1
            else:
                t_buf[i] = self.table[ s_buf[ i ] ]
        # Done
        return rval

    def __getitem__( self, int x ):
        if x == -1: return -1
        assert 0 <= x < self.in_size
        return self.table[ x ]

    def collapse( self, int a, int b ):
        cdef int i
        cdef IntToIntMapping copy
        copy = IntToIntMapping( self.in_size )
        copy.out_size = self.out_size - 1
        if a > b: a, b = b, a
        for i from 0 <= i < self.in_size:
            if self.table[i] == b: copy.table[i] = a
            elif self.table[i] == copy.out_size: copy.table[i] = b
            else: copy.table[i] = self.table[i]
        return copy

    def expand( self, int x ):
        """Grow the alphabet by making 'a' a seperate symbol. If it already mapped to a single symbol, do nothing"""
        cdef int i, count, a, b
        cdef IntToIntMapping copy
        # Get the symbol x maps to
        a = self.table[x]
        # Symbols that map to -1 should not be touched
        if a < 0: return self
        # Count how many other input symbols map to a
        count = 0
        for i from 0 <= i < self.in_size:
            if self.table[i] == a: count = count + 1
        # Already a singleton
        if count < 2: return self
        # Otherwise, make a copy with the separated symbol
        copy = IntToIntMapping( self.in_size )
        copy.out_size = self.out_size + 1
        for i from 0 <= i < self.in_size:
            copy.table[i] = self.table[i]
        copy.table[x] = self.out_size
        return copy

    def expand_out( self, int a ):
        """Grow the alphabet breaking 'a' into two symbols randomly"""
        cdef int i, count, to_split, b
        cdef IntToIntMapping copy
        count = 0
        for i from 0 <= i < self.in_size:
            if self.table[i] == a: count = count + 1
        if count < 2: return self
        copy = IntToIntMapping( self.in_size )
        copy.out_size = self.out_size + 1
        b = self.out_size
        to_split = random.randrange( count )
        count = 0
        for i from 0 <= i < self.in_size:
            if self.table[i] == a: 
                if count == to_split: copy.table[i] = b
                else: copy.table[i] = a
                count = count + 1
            else:
                copy.table[i] = self.table[i]
        return copy

    def expand_random_split( self, int a ):
        """Grow the alphabet breaking 'a' into two symbols randomly"""
        cdef int i, count, b
        cdef IntToIntMapping copy
        count = 0
        for i from 0 <= i < self.in_size:
            if self.table[i] == a: count = count + 1
        if count < 2: return self
        copy = IntToIntMapping( self.in_size )
        copy.out_size = self.out_size + 1
        b = self.out_size
        to_split = random.sample( range( count ), count/2 )
        count = 0
        for i from 0 <= i < self.in_size:
            if self.table[i] == a: 
                if count in to_split: copy.table[i] = b
                else: copy.table[i] = a
                count = count + 1
            else:
                copy.table[i] = self.table[i]
        return copy

    def get_in_size( self ): 
        return self.in_size

    def get_out_size( self ): 
        return self.out_size
    
    def get_table( self ):
        rval = zeros( self.in_size, 'i' )
        for i in range( self.in_size ):
            rval[i] = self.table[i]
        return rval





"""
T="/Users/james/cache/hg18/align/multiz28way/chr10.maf"
C="/Users/james/cache/hg18/align/multiz28way/chr10.maf.lzo"

def test():
    f = seeklzop.SeekableLzopFile( C, C + "t", block_cache_size=20 )
    for line in f:
        pass

def test_random_seeking():
    s = os.stat( T ).st_size
    raw = open( T )
    f = seeklzop.SeekableLzopFile( C, C + "t", block_cache_size=20 )
    for i in range( 1000 ):
        seek_to = random.randrange( s )

        f.seek( seek_to )
        raw.seek( seek_to )

        l1 = f.readline()
        l2 = raw.readline()

        assert l1 == l2, "%r != %r" % ( l1, l2 )
        assert raw.tell() == f.tell(), "tells not equal"
"""



#!/usr/bin/env python

"""
Read a set of ranges and a nib file, print portions of nib overlapping
those ranges to stdout

TODO: General sequence handling would be nice, as well as merging with
      'nib_intervals_to_fasta.py'.

usage: %prog nib_dir < range_file
"""
from __future__ import print_function

import sys

import bx.seq.nib
from bx.cookbook import doc_optparse


def __main__():
    options, args = doc_optparse.parse(__doc__)

    try:
        nib_dir = args[0]
    except IndexError:
        doc_optparse.exit()

    nibs = {}

    for line in sys.stdin:
        fields = line.split()
        chrom, start, end = fields[0], int(fields[1]), int(fields[2])
        print(">", chrom, start, end)
        if chrom in nibs:
            nib = nibs[chrom]
        else:
            nibs[chrom] = nib = bx.seq.nib.NibFile(open("%s/%s.nib" % (nib_dir, chrom)))
        print_wrapped(nib.get(start, end - start))


def print_wrapped(s):
    l = len(s)
    c = 0
    while c < l:
        b = min(c + 50, l)
        print(s[c:b])
        c = b


if __name__ == "__main__":
    __main__()



from numpy import (
    allclose,
    isnan
)

from . import pwm


def test_create():
    m = pwm.FrequencyMatrix.from_rows(['A', 'C', 'G', 'T'], get_ctcf_rows())
    # Alphabet sort
    assert m.sorted_alphabet == ['A', 'C', 'G', 'T']
    # Character to index mapping
    assert m.char_to_index[ord('A')] == 0
    assert m.char_to_index[ord('C')] == 1
    assert m.char_to_index[ord('G')] == 2
    assert m.char_to_index[ord('T')] == 3
    assert m.char_to_index[ord('Q')] == -1
    # Values
    assert allclose(m.values[0], [2620, 2052, 3013, 2314])
    assert allclose(m.values[19], [3144, 3231, 3056, 567])


def test_scoring():
    m = pwm.FrequencyMatrix.from_rows(['A', 'C', 'G', 'T'], get_ctcf_rows())
    # Stormo method
    sm = m.to_stormo_scoring_matrix()
    # Forward matches
    assert allclose(sm.score_string("AATCACCACCTCCTGGCAGG")[0], -156.8261261)
    assert allclose(sm.score_string("TGCCTGCCTCTGTAGGCTCC")[0], -128.8106842)
    assert allclose(sm.score_string("GTTGCCAGTTGGGGGAAGCA")[0], 4.65049839)
    assert allclose(sm.score_string("GCAGACACCAGGTGGTTCAG")[0], 1.60168743)
    # Reverse matches
    rc = sm.reverse_complement()
    assert allclose(rc.score_string("AATCACCACCTCCTGGCAGG")[0], 0.014178276062)
    assert allclose(rc.score_string("TGCCTGCCTCTGTAGGCTCC")[0], 0.723828315735)
    assert allclose(rc.score_string("GTTGCCAGTTGGGGGAAGCA")[0], -126.99407196)
    assert allclose(rc.score_string("GCAGACACCAGGTGGTTCAG")[0], -86.9560623169)
    # Nothing valid
    assert isnan(sm.score_string_with_gaps("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")).all()
    # Too short
    assert isnan(sm.score_string("TTTT")).all()


def test_scoring_with_gaps():
    m = pwm.FrequencyMatrix.from_rows(['A', 'C', 'G', 'T'], get_ctcf_rows())
    # Stormo method
    sm = m.to_stormo_scoring_matrix()
    # Forward matches
    assert allclose(sm.score_string_with_gaps("GTTGCCAGT----TGGGGGAAGCATTT---AA")[0], 4.65049839)
    assert allclose(sm.score_string_with_gaps("GCAGA--CACCAGGTGG--TTCAG---")[0], 1.60168743)
    assert allclose(sm.score_string_with_gaps("----GTTGCCAGTTGGGGGAAGCA")[4], 4.65049839)
    assert allclose(sm.score_string_with_gaps("TTT--GTT--GCCA--GTTGGGG-G-A-A-G-C-A-")[5], 4.65049839)
    assert isnan(sm.score_string_with_gaps("TTT--GTT--GCCA--GTTGGGG-G-A-A-G-C-A-")[4])
    # Nothing valid
    assert isnan(sm.score_string_with_gaps("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")).all()
    assert isnan(sm.score_string_with_gaps("------------------------------------")).all()
    # Too short
    assert isnan(sm.score_string_with_gaps("TTTT")).all()
    assert isnan(sm.score_string_with_gaps("TTTT----")).all()


def get_ctcf_rows():
    """
    The CTCF primary site motif
    """
    return [
        [2620, 2052, 3013, 2314],
        [0, 3580, 1746, 4672],
        [2008, 1790, 4497, 1703],
        [3362, 0, 6637, 0],
        [0, 10000, 0, 0],
        [0, 10000, 0, 0],
        [7467, 0, 1310, 1222],
        [786, 4890, 4323, 0],
        [1179, 6288, 829, 1703],
        [10000, 0, 0, 0],
        [0, 0, 10000, 0],
        [4847, 0, 5152, 0],
        [0, 0, 6200, 3799],
        [0, 0, 10000, 0],
        [0, 0, 10000, 0],
        [1572, 7467, 0, 960],
        [3842, 0, 5545, 611],
        [0, 5895, 4104, 0],
        [1615, 4192, 1397, 2794],
        [3144, 3231, 3056, 567]
    ]



#!/usr/bin/env python

"""
'Tile' the blocks of a maf file over each of a set of intervals. The
highest scoring block that covers any part of a region will be used, and
pieces not covered by any block filled with "-" or optionally "*". The list
of species to tile is specified by `tree` (either a tree or just a comma
separated list). The `seq_db` is a lookup table mapping chromosome names
to nib file for filling in the reference species. Maf files must be indexed.

NOTE: See maf_tile_2.py for a more sophisticated version of this program, I
      think this one will be eliminated in the future.

usage: %prog tree maf_files...
    -m, --missingData: Inserts wildcards for missing block rows instead of '-'
"""

import string
import sys

import bx.align as align
import bx.align.maf
import bx.seq.nib
from bx.cookbook import doc_optparse

tree_tx = string.maketrans("(),", "   ")


def main():

    options, args = doc_optparse.parse(__doc__)
    try:
        sources = args[0].translate(tree_tx).split()
        seq_db = load_seq_db(args[1])
        index = bx.align.maf.MultiIndexed(args[2:])

        out = bx.align.maf.Writer(sys.stdout)
        missing_data = bool(options.missingData)
    except Exception:
        doc_optparse.exception()

    for line in sys.stdin:
        ref_src, start, end = line.split()[0:3]
        do_interval(sources, index, out, ref_src, int(start), int(end), seq_db, missing_data)

    out.close()


def load_seq_db(fname):
    db = {}
    for line in open(fname):
        fields = line.split(',')
        src = fields[1] + "." + fields[2]
        seq = fields[4]
        db[src] = seq.strip()
    return db


def do_interval(sources, index, out, ref_src, start, end, seq_db, missing_data):

    assert sources[0].split('.')[0] == ref_src.split('.')[0], "%s != %s" % (sources[0].split('.')[0], ref_src.split('.')[0])

    base_len = end - start

    blocks = index.get(ref_src, start, end)
    # From low to high score
    blocks.sort(key=lambda _: _.score)

    mask = [-1] * base_len

    ref_src_size = None
    for i, block in enumerate(blocks):
        ref = block.get_component_by_src_start(ref_src)
        ref_src_size = ref.src_size
        assert ref.strand == "+"
        slice_start = max(start, ref.start)
        slice_end = min(end, ref.end)
        for j in range(slice_start, slice_end):
            mask[j-start] = i

    tiled = []
    for i in range(len(sources)):
        tiled.append([])

    for ss, ee, index in intervals_from_mask(mask):
        if index < 0:
            tiled[0].append(bx.seq.nib.NibFile(open(seq_db[ref_src])).get(start+ss, ee-ss))
            for row in tiled[1:]:
                if missing_data:
                    row.append("*" * (ee - ss))
                else:
                    row.append("-" * (ee - ss))
        else:
            slice_start = start + ss
            slice_end = start + ee
            block = blocks[index]
            ref = block.get_component_by_src_start(ref_src)
            sliced = block.slice_by_component(ref, slice_start, slice_end)
            sliced = sliced.limit_to_species(sources)
            sliced.remove_all_gap_columns()
            for i, src in enumerate(sources):
                comp = sliced.get_component_by_src_start(src)
                if comp:
                    tiled[i].append(comp.text)
                else:
                    if missing_data:
                        tiled[i].append("*" * sliced.text_size)
                    else:
                        tiled[i].append("-" * sliced.text_size)

    a = align.Alignment()
    for i, name in enumerate(sources):
        text = "".join(tiled[i])
        size = len(text) - text.count("-")
        if i == 0:
            if ref_src_size is None:
                ref_src_size = bx.seq.nib.NibFile(open(seq_db[ref_src])).length
            c = align.Component(ref_src, start, end-start, "+", ref_src_size, text)
        else:
            c = align.Component(name + ".fake", 0, size, "?", size, text)
        a.add_component(c)

    out.write(a)


def intervals_from_mask(mask):
    start = 0
    last = mask[0]
    for i in range(1, len(mask)):
        if mask[i] != last:
            yield start, i, last
            start = i
            last = mask[i]
    yield start, len(mask), last


main()



"""
Classes for index files that map genomic intervals to values.

:Authors: James Taylor (james@bx.psu.edu), Bob Harris (rsharris@bx.psu.edu)

An interval index file maps genomic intervals to values.

This implementation writes version 1 file format, and reads versions 0 and 1.

Index File Format
-----------------

All fields are in big-endian format (most significant byte first).

All intervals are origin-zero, inclusive start, exclusive end.

The file begins with an index file header, then is immediately followed
by an index table.  The index table points to index headers, and index
headers point to bins.  Index headers and bins are referenced via pointers
(file offsets), and can be placed more or less anywhere in the file.

File header
~~~~~~~~~~~

============ ===========   =================================================
offset 0x00: 2C FF 80 0A   magic number
offset 0x04: 00 00 00 01   version (00 00 00 00 is also supported)
offset 0x08: 00 00 00 2A   (N) number of index sets
offset 0x0C:  ...          index table
============ ===========   =================================================

Index table
~~~~~~~~~~~

The index table is a list of N index headers, packed sequentially and
sorted by name.  The first begins at offset 0x0C.  Each header describes
one set of intervals.

============ ===========   =================================================
offset:      xx xx xx xx   (L) length of index src name
offset+4:     ...          index src name (e.g. canFam1.chr1)
offset+4+L:  xx xx xx xx   offset (in this file) to index data
offset+8+L:  xx xx xx xx   (B) number of bytes in each value;  for version
                           0, this field is absent, and B is assumed to be 4
============ ===========   =================================================

Index data
~~~~~~~~~~

The index data for (for one index table) consists of the overall range of
intervals followed by an array of pointers to bins.  The length of the
array is 1+binForRange(maxEnd-1,maxEnd), where maxEnd is the maximum
interval end.

============ ===========   =================================================
offset:      xx xx xx xx   minimum interval start
offset+4:    xx xx xx xx   maximum interval end
offset+8:    xx xx xx xx   offset (in this file) to bin 0
offset+12:   xx xx xx xx   number of intervals in bin 0
offset+16:   xx xx xx xx   offset (in this file) to bin 1
offset+20:   xx xx xx xx   number of intervals in bin 1
...          ...           ...
============ ===========   =================================================

Bin
~~~

A bin is an array of (start,end,val), sorted by increasing start (with
end and val as tiebreakers).  Note that bins may be empty (the number of
intervals indicated in the index data is zero).  Note that B is determined
from the appropriate entry in the index table.

============ ===========   =================================================
offset:      xx xx xx xx   start for interval 1
offset+4:    xx xx xx xx   end   for interval 1
offset+8:     ...          (B bytes) value for interval 1
offset+8+B:  xx xx xx xx   start for interval 2
offset+12+B: xx xx xx xx   end   for interval 2
offset+16+B:  ...          (B bytes) value for interval 2
...          ...           ...
============ ===========   =================================================
"""
import os.path
import sys
from bisect import (
    insort,
    insort_right
)
from struct import (
    calcsize,
    pack,
    unpack
)
from warnings import warn

import six

from bx.misc import filecache

try:
    from bx.misc import seekbzip2
except ImportError:
    seekbzip2 = None

try:
    from bx.misc import seeklzop
except ImportError:
    seeklzop = None

__all__ = ['Indexes', 'Index']

MAGIC = 0x2cff800a
VERSION = 2

# These three constants determine the structure of the default binning strategy
BIN_LEVELS = 6        # Number of levels of bins to build
BIN_FIRST_SHIFT = 17  # Number of bits for the bottom level bin
BIN_NEXT_SHIFT = 3    # Number of bits for each higher level bin

# Build offset and max size arrays for each bin level
BIN_OFFSETS = [1, 0]
BIN_OFFSETS_MAX = [(1 << BIN_FIRST_SHIFT << BIN_NEXT_SHIFT), (1 << BIN_FIRST_SHIFT)]
for i in range(BIN_LEVELS - 2):
    BIN_OFFSETS.insert(0, (2 ** (3 * (i + 1))) + BIN_OFFSETS[0])
    BIN_OFFSETS_MAX.insert(0, (BIN_OFFSETS_MAX[0] << BIN_NEXT_SHIFT))
# The maximum size for the top bin is actually bigger than the signed integers
# we use to store positions in the file, so we'll change it to prevent confusion
BIN_OFFSETS_MAX[0] = sys.maxsize

# Constants for the minimum and maximum size of the overall interval
MIN = 0
OLD_MAX = 512 * 1024 * 1024  # Maximum size supported by versions < 2
DEFAULT_MAX = 512 * 1024 * 1024  # Default max size to use when none is passed
MAX = 2 ** 31  # Absolute max size (limited by file format)


def offsets_for_max_size(max_size):
    """
    Return the subset of offsets needed to contain intervals over (0,max_size)
    """
    for i, max in enumerate(reversed(BIN_OFFSETS_MAX)):
        if max_size < max:
            break
    else:
        raise Exception("%d is larger than the maximum possible size (%d)" % (max_size, BIN_OFFSETS_MAX[0]))
    return BIN_OFFSETS[(len(BIN_OFFSETS) - i - 1):]


def bin_for_range(start, end, offsets=None):
    """Find the smallest bin that can contain interval (start,end)"""
    if offsets is None:
        offsets = BIN_OFFSETS
    start_bin, end_bin = start, max(start, end - 1)
    start_bin >>= BIN_FIRST_SHIFT
    end_bin >>= BIN_FIRST_SHIFT
    for offset in offsets:
        if start_bin == end_bin:
            return offset + start_bin
        else:
            start_bin >>= BIN_NEXT_SHIFT
            end_bin >>= BIN_NEXT_SHIFT
    raise Exception("Interval (%d,%d) out of range")


class AbstractMultiIndexedAccess(object):
    """
    Allows accessing multiple indexes / files as if they were one
    """
    indexed_access_class = None

    def __init__(self, filenames, index_filenames=None, keep_open=False, use_cache=False, **kwargs):
        # TODO: Handle index_filenames argument
        self.indexes = [
            self.new_indexed_access(fname, keep_open=keep_open, use_cache=use_cache, **kwargs)
            for fname in filenames]

    def new_indexed_access(self, data_filename, index_filename=None, keep_open=False, **kwargs):
        return self.indexed_access_class(data_filename, index_filename, keep_open, **kwargs)

    def get(self, src, start, end):
        return [block for block in self.get_as_iterator(src, start, end)]

    def get_as_iterator(self, src, start, end):
        for block, index, offset in self.get_as_iterator_with_index_and_offset(src, start, end):
            yield block

    def get_as_iterator_with_index_and_offset(self, src, start, end):
        for index in self.indexes:
            for block, idx, offset in index.get_as_iterator_with_index_and_offset(src, start, end):
                yield block, idx, offset

    def close(self):
        for index in self.indexes:
            index.close()


class AbstractIndexedAccess(object):
    """Indexed access to a data using overlap queries, requires an index file"""

    def __init__(self, data_filename, index_filename=None, keep_open=False, use_cache=False, **kwargs):
        self.data_kwargs = kwargs
        self.data_filename = data_filename
        if data_filename.endswith(".bz2"):
            if seekbzip2 is None:
                raise Exception("Trying to open .bz2 file but no seekbzip2 module found")
            table_filename = data_filename + "t"
            self.table_filename = table_filename
            if not os.path.exists(table_filename):
                raise Exception("Cannot find bz2t file for: " + data_filename)
            self.file_type = "bz2t"
            # Strip .bz2 from the filename before adding ".index"
            data_filename_root = data_filename[:-4]
        elif data_filename.endswith(".lzo"):
            if seeklzop is None:
                raise Exception("Trying to open .lzo file but no seeklzop module found")
            table_filename = data_filename + "t"
            self.table_filename = table_filename
            if not os.path.exists(table_filename):
                raise Exception("Cannot find lzot file for: " + data_filename)
            self.file_type = "lzot"
            # Strip .lzo from the filename before adding ".index"
            data_filename_root = data_filename[:-4]
        else:
            self.file_type = "plain"
            data_filename_root = data_filename
        # Open index
        if index_filename is None:
            index_filename = data_filename_root + ".index"
        self.indexes = Indexes(filename=index_filename)
        # Use a file cache?
        self.use_cache = use_cache
        # Open now?
        if keep_open:
            self.f = self.open_data()
        else:
            self.f = None

    def close(self):
        if self.f:
            self.f.close()
            self.f = None

    def open_data(self):
        if self.file_type == "plain":
            return open(self.data_filename, 'rb')
        elif self.file_type == "bz2t":
            f = seekbzip2.SeekableBzip2File(self.data_filename, self.table_filename)
            if self.use_cache:
                return filecache.FileCache(f, f.size)
            else:
                return f
        elif self.file_type == "lzot":
            if self.use_cache:
                block_cache_size = 20
            else:
                block_cache_size = 0
            f = seeklzop.SeekableLzopFile(self.data_filename,
                                          self.table_filename,
                                          block_cache_size=block_cache_size)
            return f

    def get(self, src, start, end):
        return [val for val in self.get_as_iterator(src, start, end)]

    def get_as_iterator(self, src, start, end):
        for val, index, offset in self.get_as_iterator_with_index_and_offset(src, start, end):
            yield val

    def get_as_iterator_with_index_and_offset(self, src, start, end):
        for val_start, val_end, val in self.indexes.find(src, start, end):
            yield self.get_at_offset(val), self, val

    def get_at_offset(self, offset):
        if self.f:
            self.f.seek(offset)
            return self.read_at_current_offset(self.f, **self.data_kwargs)
        else:
            f = self.open_data()
            try:
                f.seek(offset)
                return self.read_at_current_offset(f, **self.data_kwargs)
            finally:
                f.close()

    def read_at_current_offset(self, file, **kwargs):
        raise TypeError("Abstract Method")


class Indexes(object):
    """A set of indexes, each identified by a unique name"""

    def __init__(self, filename=None):
        self.indexes = dict()
        if filename is not None:
            self.open(filename)

    def add(self, name, start, end, val, max=DEFAULT_MAX):
        if name not in self.indexes:
            self.indexes[name] = Index(max=max)
        self.indexes[name].add(start, end, val)

    def get(self, name):
        if self.indexes[name] is None:
            offset, value_size = self.offsets[name]
            self.indexes[name] = Index(filename=self.filename, offset=offset, value_size=value_size, version=self.version)
        return self.indexes[name]

    def find(self, name, start, end):
        if name in self.indexes:
            return self.get(name).find(start, end)
        else:
            return []

    def open(self, filename):
        self.filename = filename
        self.offsets = dict()  # (will map key to (offset,value_size))
        f = open(filename, 'rb')
        magic, version, length = read_packed(f, ">3I")
        if magic != MAGIC:
            raise Exception("File does not have expected header")
        if version > VERSION:
            warn("File claims version %d, I don't known anything about versions beyond %d. Attempting to continue", version, VERSION)
        self.version = version
        for i in range(length):
            key_len = read_packed(f, ">I")
            key = f.read(key_len).decode()
            offset = read_packed(f, ">I")
            if version == 0:
                value_size = 4
            else:
                value_size = read_packed(f, ">I")
                assert value_size % 4 == 0, "unsupported value size: %s" % value_size
            self.indexes[key] = None
            self.offsets[key] = (offset, value_size)
        f.close()

    def write(self, f):
        keys = sorted(self.indexes.keys())
        # First determine the size of the header
        base = calcsize(">3I")
        for key in keys:
            key = str(key)
            base += calcsize(">I")
            base += len(key)
            base += calcsize(">2I")
        # Now actually write the header
        write_packed(f, ">3I", MAGIC, VERSION, len(self.indexes))
        # And write the index table
        for key in keys:
            key = str(key)
            # Write the string prefixed by its length (pascal!)
            write_packed(f, ">I", len(key))
            f.write(six.b(key))
            # Write offset
            write_packed(f, ">I", base)
            base += self.indexes[key].bytes_required()
            # Write value size
            write_packed(f, ">I", self.indexes[key].value_size)
        # And finally write each index in order
        for key in keys:
            self.indexes[key].write(f)


class Index(object):

    def __init__(self, min=MIN, max=DEFAULT_MAX, filename=None, offset=0, value_size=None, version=None):
        self._value_size = value_size
        self.max_val = 1   # (1, rather than 0, to force value_size > 0)
        if filename is None:
            self.new(min, max)
        else:
            self.open(filename, offset, version)

    def get_value_size(self):
        if self._value_size is not None:
            return self._value_size
        else:
            return round_up_to_4(bytes_of(self.max_val))
    value_size = property(fget=get_value_size)

    def new(self, min, max):
        """Create an empty index for intervals in the range min, max"""
        # Ensure the range will fit given the shifting strategy
        assert MIN <= min <= max <= MAX
        self.min = min
        self.max = max
        # Determine offsets to use
        self.offsets = offsets_for_max_size(max)
        # Determine the largest bin we will actually use
        self.bin_count = bin_for_range(max - 1, max, offsets=self.offsets) + 1
        # Create empty bins
        self.bins = [[] for i in range(self.bin_count)]

    def open(self, filename, offset, version):
        self.filename = filename
        self.offset = offset
        # Open the file and seek to where we expect our header
        f = open(filename, 'rb')
        f.seek(offset)
        # Read min/max
        min, max = read_packed(f, ">2I")
        self.new(min, max)
        # Decide how many levels of bins based on 'max'
        if version < 2:
            # Prior to version 2 all files used the bins for 512MB
            self.offsets = offsets_for_max_size(OLD_MAX - 1)
        else:
            self.offsets = offsets_for_max_size(max)
        # Read bin indexes
        self.bin_offsets = []
        self.bin_sizes = []
        for i in range(self.bin_count):
            o, s = read_packed(f, ">2I")
            self.bin_offsets.append(o)
            self.bin_sizes.append(s)
        # Initialize bins to None, indicating that they need to be loaded
        self.bins = [None for i in range(self.bin_count)]

    def add(self, start, end, val):
        """Add the interval (start,end) with associated value val to the index"""
        insort(self.bins[bin_for_range(start, end, offsets=self.offsets)], (start, end, val))
        assert val >= 0
        self.max_val = max(self.max_val, val)

    def find(self, start, end):
        rval = []
        start_bin = (max(start, self.min)) >> BIN_FIRST_SHIFT
        end_bin = (min(end, self.max) - 1) >> BIN_FIRST_SHIFT
        for offset in self.offsets:
            for i in range(start_bin + offset, end_bin + offset + 1):
                if self.bins[i] is None:
                    self.load_bin(i)
                # Iterate over bin and insert any overlapping elements into return value
                for el_start, el_end, val in self.bins[i]:
                    if el_start < end and el_end > start:
                        insort_right(rval, (el_start, el_end, val))
            start_bin >>= BIN_NEXT_SHIFT
            end_bin >>= BIN_NEXT_SHIFT
        return rval

    def iterate(self):
        for i in range(self.bin_count):
            if self.bins[i] is None:
                self.load_bin(i)
            for entry in self.bins[i]:
                yield entry

    def load_bin(self, index):
        bin = []
        if self.bin_sizes[index] == 0:
            self.bins[index] = bin
            return
        f = open(self.filename, 'rb')
        f.seek(self.bin_offsets[index])
        # One big read for happy NFS
        item_size = self.value_size + calcsize(">2I")
        buffer = f.read(self.bin_sizes[index] * item_size)
        for i in range(self.bin_sizes[index]):
            start, end = unpack(">2I", buffer[i*item_size:i*item_size+8])
            val = unpack_uints(buffer[i*item_size+8:(i+1)*item_size])
            bin.append((start, end, val))
        self.bins[index] = bin
        f.close()

    def write(self, f):
        value_size = self.value_size
        item_size = value_size + calcsize(">2I")
        # Write min/max
        write_packed(f, ">2I", self.min, self.max)
        # Write table of bin sizes and offsets
        base = f.tell() + self.bin_count * calcsize(">2I")
        for bin in self.bins:
            write_packed(f, ">2I", base, len(bin))
            base += len(bin) * item_size
        # Write contents of each bin
        for bin in self.bins:
            for start, end, val in bin:
                write_packed(f, ">2I", start, end)
                write_packed_uints(f, val, value_size)

    def bytes_required(self):
        item_size = self.value_size + calcsize(">2I")
        rval = calcsize(">2I")
        rval += self.bin_count * calcsize(">2I")
        for bin in self.bins:
            rval += len(bin) * item_size
        return rval


def write_packed(f, pattern, *vals):
    f.write(pack(pattern, *vals))


def read_packed(f, pattern):
    rval = unpack(pattern, f.read(calcsize(pattern)))
    if len(rval) == 1:
        return rval[0]
    return rval


def write_packed_uints(f, v, num_bytes):
    if num_bytes < 4:
        write_packed(f, ">I", v)
    else:
        parts = []
        while num_bytes > 0:
            parts.append(v & 0xFFFFFFFF)
            v >>= 32
            num_bytes -= 4
        parts.reverse()  # (write most-significant chunk first)
        write_packed(f, ">%dI" % len(parts), *parts)


def unpack_uints(parts):
    chunks = len(parts)/4
    vals = unpack(">%dI" % chunks, parts)
    val = vals[0]
    for v in vals[1:]:
        val = (val << 32) + v
    return val


def bytes_of(v):
    assert v > 0
    b = 0
    while v > 0:
        v >>= 8
        b += 1
    return b


def round_up_to_4(v):
    if v % 4 == 0:
        return v
    else:
        return v + 4 - (v % 4)



"""
An ASCII text progress bar. See __main__ for command line use (using \r to
move the cursor back to the start of the current line is the key, on
terminals that do not support this functionality the progress bar will
not work as well).

http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/168639
"""
from __future__ import print_function

import sys


class ProgressBar(object):
    def __init__(self, minValue=0, maxValue=10, totalWidth=72):
        self.progBar = "[]"   # This holds the progress bar string
        self.min = minValue
        self.max = maxValue
        self.span = maxValue - minValue
        self.width = totalWidth
        self.amount = 0       # When amount == max, we are 100% done
        self.update(0)  # Build progress bar string

    def update(self, newAmount=0):
        if newAmount < self.min:
            newAmount = self.min
        if newAmount > self.max:
            newAmount = self.max
        self.amount = newAmount

        # Figure out the new percent done, round to an integer
        diffFromMin = float(self.amount - self.min)
        percentDone = (diffFromMin / float(self.span)) * 100.0
        percentDone = round(percentDone)
        percentDone = int(percentDone)

        # Figure out how many hash bars the percentage should be
        allFull = self.width - 2
        numHashes = (percentDone / 100.0) * allFull
        numHashes = int(round(numHashes))

        # build a progress bar with hashes and spaces
        if allFull == numHashes:
            self.progBar = "[" + '='*(numHashes) + "]"
        else:
            self.progBar = "[" + '='*(numHashes-1) + '>' + ' '*(allFull-numHashes) + "]"

        # figure out where to put the percentage, roughly centered
        percentPlace = (len(self.progBar) / 2) - len(str(percentDone))
        percentString = str(percentDone) + "%"

        # slice the percentage into the bar
        self.progBar = self.progBar[0:percentPlace] + percentString + self.progBar[percentPlace+len(percentString):]

    def update_and_print(self, newAmount=0, f=sys.stdout):
        self.update(newAmount)
        print("\r", self, end=' ', file=f)
        f.flush()

    def __str__(self):
        return str(self.progBar)


def iterprogress(sized_iterable):
    """
    Iterate something printing progress bar to stdout
    """
    pb = ProgressBar(0, len(sized_iterable))
    for i, value in enumerate(sized_iterable):
        yield value
        pb.update_and_print(i, sys.stderr)


if __name__ == "__main__":
    bar = ProgressBar(0, 1000, 80)

    for i in range(1000):
        bar.update(i)
        print("\r", bar, end=' ')
        sys.stdout.flush()

    print()



from six.moves import reduce
from six.moves.collections_abc import Mapping

from bx.misc.binary_file import (
    BinaryFileReader,
    BinaryFileWriter
)


def cdbhash(s):
    return reduce(lambda h, c: (((h << 5) + h) ^ ord(c)) & 0xffffffff, s, 5381)


class FileCDBDict(Mapping):
    """
    For accessing a CDB structure on disk. Read only. Currently only supports
    access by key (getitem).

    NOTE: The keys method could be implemented by scanning the main table.
    """

    def __init__(self, file, is_little_endian=True):
        # TODO: Deal with endianess
        self.io = BinaryFileReader(file, is_little_endian=is_little_endian)
        self.header_offset = self.io.tell()
        # Read the whole header (only 2k)
        self.header = [(self.io.read_uint32(), self.io.read_uint32()) for _ in range(256)]

    def __getitem__(self, key):
        hash = cdbhash(key)
        # Find position of subtable using 8 LSBs of hash
        subtable_offset = self.header[hash % 256][0]
        subtable_size = self.header[hash % 256][1]
        if subtable_size == 0:
            raise KeyError
        # Seek into subtable and look for match
        start = (hash >> 8)
        for i in range(subtable_size):
            offset = subtable_offset + ((start + i) % subtable_size) * 8
            self.io.seek(offset)
            h = self.io.read_uint32()
            p = self.io.read_uint32()
            # Hit an empty bin, no match for key
            if p == 0:
                raise KeyError
            # Hash matches, need to check full key
            if h == hash:
                self.io.seek(p)
                klen = self.io.read_uint32()
                vlen = self.io.read_uint32()
                k = self.io.read(klen).decode()
                if k == key:
                    v = self.io.read(vlen).decode()
                    return v
        else:
            # Visited every slot and no match (should never happen since
            # there are empty slots by contruction)
            raise KeyError

    def __iter__(self):
        raise NotImplementedError()

    def __len__(self):
        raise NotImplementedError()

    @classmethod
    def to_file(Class, dict, file, is_little_endian=True):
        """
        For constructing a CDB structure in a file. Able to calculate size on
        disk and write to a file
        """
        io = BinaryFileWriter(file, is_little_endian=is_little_endian)
        start_offset = io.tell()
        # Header is of fixed length
        io.seek(start_offset + (8 * 256))
        # For each item, key and value length (written as length prefixed
        # strings). We also calculate the subtables on this pass.
        # NOTE: This requires the key and value be byte strings, support for
        #       dealing with encoding specific value types should be
        #       added to this wrapper
        subtables = [[] for i in range(256)]
        for key, value in dict.items():
            pair_offset = io.tell()
            io.write_uint32(len(key))
            io.write_uint32(len(value))
            io.write(key)
            io.write(value)
            hash = cdbhash(key)
            subtables[hash % 256].append((hash, pair_offset))
        # Save the offset where the subtables will start
        subtable_offset = io.tell()
        # Write subtables
        for subtable in subtables:
            if len(subtable) > 0:
                # Construct hashtable to be twice the size of the number
                # of items in the subtable, and built it in memory
                ncells = len(subtable) * 2
                cells = [(0, 0) for i in range(ncells)]
                for hash, pair_offset in subtable:
                    index = (hash >> 8) % ncells
                    while cells[index][1] != 0:
                        index = (index + 1) % ncells
                    # Guaranteed to find a non-empty cell
                    cells[index] = (hash, pair_offset)
                # Write subtable
                for hash, pair_offset in cells:
                    io.write_uint32(hash)
                    io.write_uint32(pair_offset)
        # Go back and write the header
        end_offset = io.tell()
        io.seek(start_offset)
        index = subtable_offset
        for subtable in subtables:
            io.write_uint32(index)
            io.write_uint32(len(subtable * 2))
            # For each cell in the subtable, a hash and a pointer to a value
            index += (len(subtable) * 2) * 8
        # Leave fp at end of cdb
        io.seek(end_offset)



#!/usr/bin/env python2.4

"""
Complement the regions of a bed file. Requires a file that maps source names
to sizes. This should be in the simple LEN file format (each line contains
a source name followed by a size, separated by whitespace).

usage: %prog bed_file chrom_length_file
"""

from __future__ import print_function

from bx.bitset_builders import binned_bitsets_from_file
from bx.cookbook import doc_optparse


def read_len(f):
    """Read a 'LEN' file and return a mapping from chromosome to length"""
    mapping = dict()
    for line in f:
        fields = line.split()
        mapping[fields[0]] = int(fields[1])
    return mapping


options, args = doc_optparse.parse(__doc__)
try:
    in_fname, len_fname = args
except Exception:
    doc_optparse.exit()

bitsets = binned_bitsets_from_file(open(in_fname))

lens = read_len(open(len_fname))

for chrom in lens:
    if chrom in bitsets:
        bits = bitsets[chrom]
        bits.invert()
        len = lens[chrom]
        end = 0
        while True:
            start = bits.next_set(end)
            if start == bits.size:
                break
            end = bits.next_clear(start)
            if end > len:
                end = len
            print("%s\t%d\t%d" % (chrom, start, end))
            if end == len:
                break
    else:
        print("%s\t%d\t%d" % (chrom, 0, lens[chrom]))



#!/usr/bin/env python

"""
Writes compressed data from a wiggle file by chromosome.

usage: %prog score_file < wiggle_data
"""
from __future__ import division, print_function

import sys

import bx.wiggle
from bx.binned_array import BinnedArray
from bx.cookbook import doc_optparse


def main():
    options, args = doc_optparse.parse(__doc__)

    scores = {}
    for i, (chrom, pos, val) in enumerate(bx.wiggle.Reader(open(sys.argv[1]))):
        if chrom not in scores:
            scores[chrom] = BinnedArray()
        scores[chrom][pos] = val

        # Status
        if i % 10000 == 0:
            print(i, "scores processed")

    for chr in scores.keys():
        out = open(chr, "w")
        scores[chr].to_file(out)
        out.close()


if __name__ == "__main__":
    main()



"""
Tests for `bx.bitset`.
"""

import unittest

import bx.bitset


class AbstractTests(object):

    def assert_bits(self, bits, list):
        assert bits.size == len(list), "Bitset size and verification list size do not match"
        for i in range(bits.size):
            self.assertEqual(bits[i], list[i])

    def test_overflow_create(self):
        self.assertRaises(ValueError, self.new_bits, 4000000000)

    def test_overflow_access(self):
        bits = self.new_bits(100)
        self.assertRaises(IndexError, bits.set, -5)
        self.assertRaises(IndexError, bits.set, 110)

    def test_access(self):
        # Create and assert empty
        bits = self.new_bits(100)
        l = [0] * 100
        self.assert_bits(bits, l)
        # Set some positions
        for pos in (11, 14, 70, 16):
            bits.set(pos)
            l[pos] = 1
        # Clear some positions
        for pos in (14, 80, 16):
            bits.clear(pos)
            l[pos] = 0
        self.assert_bits(bits, l)

    def test_range_access(self):
        # Create and assert empty
        bits = self.new_bits(100)
        l = [0] * 100
        self.assert_bits(bits, l)
        # Set some positions
        for b, e in ((11, 14), (20, 75), (90, 99)):
            bits.set_range(b, e-b)
            for pos in range(b, e):
                l[pos] = 1
        self.assert_bits(bits, l)

    def test_count(self):
        # Create and assert empty
        bits = self.new_bits(100)
        # Set some positions
        for b, e in ((11, 14), (20, 75), (90, 100)):
            bits.set_range(b, e-b)
        self.assertEqual(bits.count_range(0, 0), 0)
        self.assertEqual(bits.count_range(0, 20), 3)
        self.assertEqual(bits.count_range(25, 25), 25)
        self.assertEqual(bits.count_range(80, 20), 10)
        self.assertEqual(bits.count_range(0, 100), 68)

    def test_find(self):
        # Create and assert empty
        bits = self.new_bits(100)
        # Set some positions
        for b, e in ((11, 14), (20, 75), (90, 100)):
            bits.set_range(b, e-b)
        # Next set
        self.assertEqual(bits.next_set(0), 11)
        self.assertEqual(bits.next_set(13), 13)
        self.assertEqual(bits.next_set(15), 20)
        # Next clear
        self.assertEqual(bits.next_clear(0), 0)
        self.assertEqual(bits.next_clear(11), 14)
        self.assertEqual(bits.next_clear(20), 75)
        self.assertEqual(bits.next_clear(92), 100)

    def test_and(self):
        bits1 = self.new_bits(100)
        bits2 = self.new_bits(100)
        bits1.set_range(20, 40)
        bits2.set_range(50, 25)
        bits1.iand(bits2)
        l = [0]*100
        for i in range(50, 60):
            l[i] = 1
        self.assert_bits(bits1, l)

    def test_or(self):
        bits1 = self.new_bits(100)
        bits2 = self.new_bits(100)
        bits1.set_range(20, 40)
        bits2.set_range(50, 25)
        bits1.ior(bits2)
        l = [0]*100
        for i in range(20, 75):
            l[i] = 1
        self.assert_bits(bits1, l)

    def test_not(self):
        bits = self.new_bits(100)
        bits.set_range(20, 40)
        bits.invert()
        l = [1]*100
        for i in range(20, 60):
            l[i] = 0
        self.assert_bits(bits, l)


class BitSetTests(AbstractTests, unittest.TestCase):
    def new_bits(self, size):
        return bx.bitset.BitSet(size)


class BinnedBitSetTests(AbstractTests, unittest.TestCase):
    def new_bits(self, size):
        granularity = size % 11
        return bx.bitset.BinnedBitSet(size, granularity)



#!/usr/bin/env python

"""
Print number of bases covered by all intervals in a bed file (bases covered by
more than one interval are counted only once). Multiple bed files can be
provided on the command line or to stdin.

usage: %prog bed files ...
"""
from __future__ import print_function

import fileinput
import sys

from bx.bitset_builders import binned_bitsets_from_file

bed_filenames = sys.argv[1:]
if bed_filenames:
    input = fileinput.input(bed_filenames)
else:
    input = sys.stdin

bitsets = binned_bitsets_from_file(input)

total = 0
for chrom in bitsets:
    total += bitsets[chrom].count_range(0, bitsets[chrom].size)

print(total)



#!/usr/bin/env python

"""
Extract ranges of scores from a sorted file in which each line contains a
position followed by a score.

TODO: The finder class might actually be useful, it strides through a file
      and builds an index based on the first line. Maybe move it into the
      library and get rid of this very specific script?

usage: %prog start_pos stop_pos
"""
from __future__ import print_function

import sys

max_cats = 1000


class Finder(object):
    def __init__(self, file, segments):
        self.file = file
        self.segments = segments
        self.make_index()

    def make_index(self):
        self.values = []
        self.positions = []

        file.seek(0, 2)
        end = file.tell()

        step = end / (self.segments - 1)

        for i in range(0, self.segments - 1):
            file.seek(i * step, 0)
            file.readline()
            position = file.tell()
            fields = file.readline().split()
            self.values.append(int(fields[0]))
            self.positions.append(position)

    def scores_in_range(self, start, end):
        position = self.positions[-1]
        for i in range(1, len(self.values)):
            if self.values[i] > start:
                position = self.positions[i - 1]
                break
        self.file.seek(position, 0)
        result = []
        while True:
            line = file.readline()
            if line == "":
                break
            fields = line.split()

            pos = int(fields[0])

            if pos < start:
                continue
            if pos > end:
                break

            result.append((pos, fields[1]))

        return result


file = open(sys.argv[1])

finder = Finder(file, 100)

scores = finder.scores_in_range(int(sys.argv[2]), int(sys.argv[3]))

rng = scores[-1][0] - scores[0][0]

if rng > max_cats:
    stride = rng // max_cats
else:
    stride = 1

for score in scores:
    if score[0] % stride == 0:
        print(score[0], score[1])
